# -*- coding: utf-8 -*-

from tkinter import END

import h5py
import matplotlib.pyplot as plt
import numpy as np

import ..utils.aux_func as aux_func
from .baseplot import Plotting
from .diagnostic import Diagnostic


class DiagTimetracesgene3d(Diagnostic):
    def __init__(self, avail_vars=None, specnames=None):
        super().__init__()
        self.name = 'Time Traces'
        self.tabs = ['GENE3D']
        self.help_txt = """Time traces of different quantities
        \n
        \nx ind : comma separated radial interval in x/a (e.g. 0.4,0.6), default all range
        \ny ind : comma separated y interval in terms of larmor radious (e.g. -10,10),
        default all range
        \nky ind : comma separated |ky| fourier modes (e.g 0.5,1.0), default 0.0, 0.0
        \nz ind : comma separated z interval (e.g., -3,3), default all range
        \nLog : log scale, default true
        \nSave h5 : save hdf5 file, only for time average plot (def. True)"""
        self.avail_vars = avail_vars
        self.specnames = specnames

        self.opts = {"quant": {'tag': "Quant", 'values': None, 'files': ['mom', 'field']},
                     "spec": {'tag': "Spec", 'values': self.list_specnames()},
                     "xind": {'tag': "x ind", 'values': None},
                     "yind": {'tag': "y ind", 'values': None},
                     "zind": {'tag': "z ind", 'values': None},
                     "kyind": {'tag': "ky ind", 'values': None},
                     "log": {'tag': "Log", 'values': [False,True]},
                     "save_h5": {'tag': 'Save h5', 'values': [True, False]}}
        self.set_defaults()

        self.options_gui = Diagnostic.OptionsGUI()

    def set_options(self, run_data, specnames=None):

        if self.opts['quant']['value'] is None:
            raise RuntimeError("No quantities given for contours")

        # radial position, unset or -1 means all
        xind = self.opts['xind']['value']
        if not xind or xind == -1:
            self.xind = 0, 100  # this will choose the minimum and maximum values
        else:
            self.xind = xind.split(',')
        self.xind_min, self.xind_max = aux_func.find_nearest_points(run_data.spatialgrid.x_a, float(self.xind[0]), float(self.xind[1]))

        # y position, unset or -1 means all
        yind = self.opts['yind']['value']
        if not yind or yind == -1:
            self.yind = -1000, 1000  # this will choose the minimum and maximum values
        else:
            self.yind = yind.split(',')
        self.yind_min, self.yind_max = aux_func.find_nearest_points(run_data.spatialgrid.y, float(self.yind[0]), float(self.yind[1]))

        kyind = self.opts['kyind']['value']
        if not kyind or kyind == -1:
            self.kyind = 0.0, 0.0
        else:
            self.kyind = kyind.split(',')
        self.kyind_min = aux_func.find_nearest(run_data.spatialgrid.ky, float(self.kyind[0]))
        self.kyind_max = aux_func.find_nearest(run_data.spatialgrid.ky, float(self.kyind[1]))

        # z position, unset or -1 means all
        zind = self.opts['zind']['value']
        if not zind or zind == -1:
            self.zind = -100, 100  # this will choose the minimum and maximum values
        else:
            self.zind = zind.split(',')
        self.zind_min, self.zind_max = aux_func.find_nearest_points(run_data.spatialgrid.z, float(self.zind[0]), float(self.zind[1]))

        self.myquant = self.opts['quant']['value']

        self.run_data = run_data
        self.get_spec_from_opts()
        self.geom = run_data.geometry
        self.get_needed_vars()

        self.time_traces = {}
        self.time_traces_ky = {}

        for file in self.needed_vars:
            # loop over quaitites in that file
            for quant in self.needed_vars[file]:
                if self.needed_vars[file][quant]:
                    if file == 'field':
                        self.time_traces[quant] = []
                        self.time_traces_ky[quant] = []
                    else:
                        for spec in self.specnames:
                            self.time_traces[quant + '#' + spec] = []
                            self.time_traces_ky[quant + '#' + spec] = []

        return self.needed_vars

    def execute(self, data, step, last_step):
        """ not much to do other than appending data """

        # loop over files
        for file in self.needed_vars.keys():
            # loop over quaitites in that file
            for quant in self.needed_vars[file].keys():
                # loop over quaitites in that file
                if self.needed_vars[file][quant]:
                    if file == 'field':
                        # no species dependency
                        data_in = getattr(getattr(data, file), quant)(step.time, getattr(step, file))

                        data_t = np.average(data_in[self.xind_min:self.xind_max+1, self.yind_min:self.yind_max+1,
                                               self.zind_min:self.zind_max+1], weights=self.geom.jacobian.T[self.xind_min:self.xind_max+1, self.yind_min:self.yind_max+1,
                                                       self.zind_min:self.zind_max+1])

                        temp = np.absolute(np.fft.fftshift(np.fft.fft(data_in, axis=1), axes=1)/self.run_data.pnt.ny0)**2
                        jac_xz = np.mean(self.geom.jacobian.T, axis=1)
                        jaco3d_xz = np.broadcast_to(jac_xz[:, np.newaxis, :], (self.run_data.pnt.nx0, self.run_data.pnt.ny0, self.run_data.pnt.nz0))

                        data_ky = np.average(temp[self.xind_min:self.xind_max+1, :, self.zind_min:self.zind_max+1],
                                    weights=jaco3d_xz[self.xind_min:self.xind_max+1, :, self.zind_min:self.zind_max+1], axis=(0, 2))


                        self.time_traces[quant].append(data_t)
                        self.time_traces_ky[quant].append(data_ky)

                    else:
                        # spec dependent
                        for spec in self.specnames:

                            data_in = getattr(getattr(data, file + '_' + spec), quant)(step.time, getattr(step, file))

                            data_t = np.average(data_in[self.xind_min:self.xind_max+1, self.yind_min:self.yind_max+1,
                                               self.zind_min:self.zind_max+1], weights=self.geom.jacobian.T[self.xind_min:self.xind_max+1, self.yind_min:self.yind_max+1,
                                                       self.zind_min:self.zind_max+1])

                            temp = np.absolute(np.fft.fftshift(np.fft.fft(data_in, axis=1), axes=1)/self.run_data.pnt.ny0)**2
                            jac_xz = np.mean(self.geom.jacobian.T, axis=1)
                            jaco3d_xz = np.broadcast_to(jac_xz[:, np.newaxis, :], (self.run_data.pnt.nx0, self.run_data.pnt.ny0, self.run_data.pnt.nz0))

                            data_ky = np.average(temp[self.xind_min:self.xind_max+1, :, self.zind_min:self.zind_max+1],
                                    weights=jaco3d_xz[self.xind_min:self.xind_max+1, :, self.zind_min:self.zind_max+1], axis=(0, 2))

                            self.time_traces[quant + '#' + spec].append(data_t)
                            self.time_traces_ky[quant + '#' + spec].append(data_ky)


    def plot(self, time_requested, output=None, out_folder=None):

        if output:
            output.info_txt.insert(END, "Time traces:\n")

        self.plotbase = Plotting()
        self.plotbase.titles.update(
                {"phi": r"$\phi$", "n": r"$n$", "u_par": r"$u_{\parallel}$", "T_par": r"$T_{\parallel}$",
                 "T_per": r"$T_{\perp}$", "Q_es": r"$Q_{es}$", "Q_em": r"$Q_{em}$",
                 "Gamma_es": r"$\Gamma_{es}$", "Gamma_em": r"$\Gamma_{em}$"})

        x_lbl = r'$t\; [c_{ref}/L_{ref}]$'

            # loop over file
        for quant in self.time_traces:

            ind_str = quant.find('#')
            if ind_str == -1:
                ttl = self.plotbase.titles[quant]
            else:
                ttl = self.plotbase.titles[quant[0:ind_str]] + " " + quant[ind_str + 1:]

            fig = plt.figure(figsize=(15, 10), dpi=100)

            text_x = 'Avg. over x = [' + '{:.1f}'.format(self.run_data.spatialgrid.x_a[self.xind_min]) + ',' + '{:.1f}'.format(self.run_data.spatialgrid.x_a[self.xind_max-1])+ ']  '
            text_y = 'y = [' + '{:.1f}'.format(self.run_data.spatialgrid.y[self.yind_min]) + ',' + '{:.1f}'.format(self.run_data.spatialgrid.y[self.yind_max-1])+ ']  '
            text_z = 'z = [' + '{:.1f}'.format(self.run_data.spatialgrid.z[self.zind_min]) + ',' + '{:.1f}'.format(self.run_data.spatialgrid.z[self.zind_max-1])+ ']'

            if self.opts['save_h5']['value']:
                file = h5py.File(out_folder / 'time_traces_{}.h5'.format(quant), 'w')
                file["/time"] = time_requested
                file["/ky"] = self.run_data.spatialgrid.ky
                file["/"+self.myquant] = self.time_traces[quant]
                file["/"+self.myquant + '_ky'] = self.time_traces_ky[quant]
                file.close()

            ax_1 = fig.add_subplot(1, 2, 1)
            if self.opts['log']['value']:
                ax_1.semilogy(time_requested, self.time_traces[quant])
            else:
                ax_1.plot(time_requested, self.time_traces[quant])
            average_quant = np.average(self.time_traces[quant])
            text_avg = ' Avg. ' + ttl +f' over time window = {average_quant:.2f}'
            ax_1.axhline(y=average_quant, color='k', linestyle=':')
            ax_1.set_xlabel(x_lbl)
            ax_1.set_title(ttl)

            ax_1 = fig.add_subplot(1, 2, 2)
            y = np.array(self.time_traces_ky[quant])
            for ky in range(0, len(y[0, :])):
                if (np.absolute(self.run_data.spatialgrid.ky[ky]) <= np.absolute(self.run_data.spatialgrid.ky[self.kyind_max])
                       and np.absolute(self.run_data.spatialgrid.ky[ky]) >= np.absolute(self.run_data.spatialgrid.ky[self.kyind_min])):
                    if self.opts['log']['value']:
                        ax_1.semilogy(time_requested, np.absolute(y[:, ky]), label=r'$k_y=$'+ str(self.run_data.spatialgrid.ky[ky]))
                    else:
                        ax_1.plot(time_requested, np.absolute(y[:, ky]), label=r'$k_y=$'+ str(self.run_data.spatialgrid.ky[ky]))
                    ax_1.set_xlabel(x_lbl)
                    ax_1.set_title(r'$|$' + ttl +'$|^2$')
                    ax_1.legend()

            fig.suptitle(text_x + text_y + text_z + text_avg)
            fig.show()



""" Module with the container objects defining velocitry grid and weights"""
# -*- coding: utf-8 -*-
import numpy as np
from .par_io import Parameters

class SpatialGrid:
    """ The numerical grid based on the parameters of a GENE run

    :param pnt: NamedTuple of the run parameters
    """

    def __init__(self, parameters):
        if isinstance(parameters, Parameters):
            pnt = parameters.asnamedtuple()
        elif isinstance(parameters, tuple):
            pnt = parameters
        
        try:
            kx_center = pnt.kx_center
        except AttributeError:
            kx_center = 0

        self._calc_kxgrid(pnt.x_local, pnt.y_local, pnt.lx, pnt.nx0, kx_center)
        self._calc_xgrid(pnt.x_local, pnt.lx, pnt.nx0, pnt.rhostar, pnt.x0)
        self._calc_kygrid(pnt)
        self._calc_ygrid(pnt)
        self.z = np.array([-np.pi + 2*np.pi*k/pnt.nz0
                           for k in range(pnt.nz0)], dtype=np.float64)
        self.nx0 = pnt.nx0
        if not pnt.y_local:
            self.ny0 = pnt.ny0
        self.nz0 = pnt.nz0

    def _calc_xgrid(self, x_local, lx, nx0, rhostar, x0):
        if x_local:
            xgrid = np.fft.fftfreq(nx0, d=1.0/lx)
            self.x = np.fft.fftshift(xgrid)
        else:
            self.x = np.array([-lx/2. + lx/(nx0 - 1)*i for i in range(0, nx0)])
            self.x_a = self.x*rhostar + x0

    def _calc_kxgrid(self, x_local, y_local, lx, nx0, kx_center):
        self.kxmin = 2*np.pi/lx

        if x_local:
            if y_local:
                kxgridp = [self.kxmin*i for i in range(int((nx0 + 1)/2) + (1 - nx0%2))]
                kxgridn = [self.kxmin*i for i in range(int(-(nx0 - 1)/2), 0)]
                # positive kx modes
                self.kx_pos = np.array(kxgridp) + kx_center
                self.kx_fftorder = np.array(kxgridp + kxgridn, dtype=np.float64) + kx_center
                # ordered kx array
                self.kx = np.array(kxgridn + kxgridp, dtype=np.float64) + kx_center
            else:
                self.kx = np.array([self.kxmin*i for i in range(nx0)])
        else:

            self.kx_fftorder = 2*np.pi*np.fft.fftfreq(nx0, d=lx/nx0)
            
            self.kx = np.fft.ifftshift(self.kx_fftorder)
            self.kx_pos = self.kx_fftorder[:int(nx0/2 + 1)]

    def _calc_ygrid(self, pnt):
        ly = 2*np.pi/pnt.kymin
        if pnt.y_local:
            self.y = np.fft.fftshift(np.fft.fftfreq(2*pnt.nky0, d=1/ly))
        else:
            if pnt.x_local:
                self.y = np.array([-ly/2. + ly/pnt.nky0*i for i in range(0, pnt.nky0)])
            else:
                self.y = np.array([ly/pnt.ny0*i for i in range(0, pnt.ny0)])

    def _calc_kygrid(self, pnt):
        if pnt.y_local:
            self.ky = np.array([pnt.kymin*(j + pnt.ky0_ind) for j in range(pnt.nky0)],
                               dtype=np.float64)
        else:
            ly = 2*np.pi/pnt.kymin
            if pnt.x_local:
                self.ky = 2*np.pi*np.fft.ifftshift(np.fft.fftfreq(pnt.nky0, d=ly/pnt.nky0))
            else:
                self.ky = np.fft.fftfreq(pnt.ny0)*pnt.ny0*pnt.kymin

    def kx_grid_slice(self, kxslice):
        """Get a full kx grid in FFT order from a slice for positive kx"""
        sliced_kxpos = self.kx_pos[kxslice]
        if self.nx0 % 2 == 1:
            sliced_kxneg = -sliced_kxpos[-1::-1]
        else:
            if kxslice.stop is None:  # Nyquist mode must be ignored
                sliced_kxneg = -sliced_kxpos[-2::-1]
            else:
                sliced_kxneg = -sliced_kxpos[-1::-1]
        if kxslice.start is None:
            sliced_kxneg = sliced_kxneg[:-1]
        sliced_kx = np.concatenate((sliced_kxpos, sliced_kxneg))
        return sliced_kx



# -*- coding: utf-8 -*-

from tkinter import END

import h5py
import matplotlib.pyplot as plt
from matplotlib.backends.backend_pdf import PdfPages
import numpy as np

import ..utils.aux_func as aux_func
import ..utils.averages as averages
from .baseplot import Plotting
from .diagnostic import Diagnostic


class DiagSlicesgene3d(Diagnostic):
    def __init__(self, avail_vars=None, specnames=None):
        super().__init__()
        self.name = 'Slices'
        self.tabs = ['GENE3D']
        self.help_txt = """Slices in different dimensions
        \n
        \nx ind : comma separated radial interval in x/a (e.g. 0.4,0.6), default all range
        \ny ind : comma separated y interval in terms of larmor radious (e.g. -10,10),
        default all range
        \nz ind : comma separated z interval (e.g., -3,3), default all range
        \nFourier x : spectral in x direction (def. False)
        \nFourier y : spectral in y direction (def. False)
        \nSquare value : square quantity (def. False)
        \nAbs. value : absolute value (def. False)
        \nt avg : time averaged (def. False)
        \nSave h5 : save hdf5 file (def. True)
        \nSave pdf : save pdf file (def. False)"""

        self.avail_vars = avail_vars
        self.specnames = specnames

        self.opts = {"quant": {'tag': "Quant", 'values': None, 'files': ['mom', 'field']},
                     "spec": {'tag': "Spec", 'values': self.list_specnames()},
                     "xind": {'tag': "x ind", 'values': None},
                     "yind": {'tag': "y ind", 'values': None},
                     "zind": {'tag': "z ind", 'values': None},
                     "f_x": {'tag': "Fourier x", 'values': [False, True]},
                     "f_y": {'tag': "Fourier y", 'values': [False, True]},
                     "abs": {'tag': "Abs. value", 'values': [False, True]},
                     "square": {'tag': "square value", 'values': [False, True]},
                     't_avg': {'tag': "t avg", 'values': [False, True]},
                     'save_h5': {'tag': "Save h5", 'values': [True, False]},
                     "save_pdf": {'tag': 'Save pdf', 'values': [False, True]}}

        self.set_defaults()

        self.options_gui = Diagnostic.OptionsGUI()

    def set_options(self, run_data, specnames=None):

        if self.opts['quant']['value'] is None:
            raise RuntimeError("No quantities given for contours")

        # radial position, unset or -1 means all
        xind = self.opts['xind']['value']
        if not xind or xind == -1:
            self.xind = 0, 100  # this will choose the minimum and maximum values
        else:
            self.xind = xind.split(',')
        self.xind_min, self.xind_max = aux_func.find_nearest_points(run_data.spatialgrid.x_a, float(self.xind[0]), float(self.xind[1]))

        # y position, unset or -1 means all
        yind = self.opts['yind']['value']
        if not yind or yind == -1:
            self.yind = -1000, 1000  # this will choose the minimum and maximum values
        else:
            self.yind = yind.split(',')
        self.yind_min, self.yind_max = aux_func.find_nearest_points(run_data.spatialgrid.y, float(self.yind[0]), float(self.yind[1]))

        # z position, unset or -1 means all
        zind = self.opts['zind']['value']
        if not zind or zind == -1:
            self.zind = -100, 100  # this will choose the minimum and maximum values
        else:
            self.zind = zind.split(',')
        self.zind_min, self.zind_max = aux_func.find_nearest_points(run_data.spatialgrid.z, float(self.zind[0]), float(self.zind[1]))
        # Fourier in x
        self.x_fourier = self.opts['f_x']['value']

        # Fourier in y
        self.y_fourier = self.opts['f_y']['value']

        self.run_data = run_data

        self.get_spec_from_opts()

        self.geom = run_data.geometry

        self.get_needed_vars()

        self.slice_xy = {}
        self.slice_xz = {}
        self.slice_yz = {}
        self.slice_x = {}
        self.slice_y = {}
        self.slice_z = {}

        for file in self.needed_vars:
            # loop over quaitites in that file
            for quant in self.needed_vars[file].keys():
                if self.needed_vars[file][quant]:
                    if file == 'field':
                        self.slice_xy[quant] = []
                        self.slice_xz[quant] = []
                        self.slice_yz[quant] = []
                        self.slice_x[quant] = []
                        self.slice_y[quant] = []
                        self.slice_z[quant] = []
                    else:
                        for spec in self.specnames:
                            self.slice_xz[quant + '#' + spec] = []
                            self.slice_xy[quant + '#' + spec] = []
                            self.slice_yz[quant + '#' + spec] = []
                            self.slice_x[quant + '#' + spec] = []
                            self.slice_y[quant + '#' + spec] = []
                            self.slice_z[quant + '#' + spec] = []

        return self.needed_vars

    def execute(self, data, step):
        """ not much to do other than appending data """

        # loop over files
        for file in self.needed_vars:
            # loop over quaitites in that file
            for quant in self.needed_vars[file]:
                # loop over quaitites in that file
                if self.needed_vars[file][quant]:
                    if file == 'field':
                        # no species dependency
                        data_in = getattr(getattr(data, file), quant)(step.time, getattr(step, file))

                        if self.x_fourier:
                            if self.y_fourier:
                                data_xyz = np.fft.fftshift(np.fft.fft2(data_in, axes=(0, 1))*np.conj(np.fft.fft2(self.geom.jacobian.T, axes=(0, 1))), axes=(0, 1))/np.mean(self.geom.jacobian.T)/self.run_data.pnt.nx0**2/self.run_data.pnt.ny0**2
                            else:
                                data_xyz = np.fft.fftshift(np.fft.fft(data_in, axis=0)*np.conj(np.fft.fft(self.geom.jacobian.T, axis=0)), axis=0)/np.mean(self.geom.jacobian.T)/self.run_data.pnt.nx0**2
                        else:
                            if self.y_fourier:
                                data_xyz = np.fft.fftshift(np.fft.fft(data_in, axis=1)*np.conj(np.fft.fft(self.geom.jacobian.T, axis=1)), axis=1)/np.mean(self.geom.jacobian.T)/self.run_data.pnt.ny0**2
                            else:
                                data_xyz = data_in*self.geom.jacobian.T/np.mean(self.geom.jacobian.T)

                        if self.opts['square']['value']: data_xyz = np.absolute(data_xyz)**2

                        data_xy = np.average(data_xyz[:, :, self.zind_min:self.zind_max+1], axis=2)
                        data_xz = np.average(data_xyz[:, self.yind_min:self.yind_max+1, :], axis=1)
                        data_yz = np.average(data_xyz[self.xind_min:self.xind_max+1, :, :], axis=0)

                        data_x = np.average(data_xyz[:, self.yind_min: self.yind_max+1, self.xind_min:self.xind_max+1], axis=(1, 2))
                        data_y = np.average(data_xyz[self.xind_min:self.xind_max+1, :, self.zind_min:self.zind_max+1], axis=(0, 2))
                        data_z = np.average(data_xyz[self.xind_min:self.xind_max+1, self.yind_min:self.yind_max+1, :], axis=(0, 1))

                        self.slice_xy[quant].append(data_xy)
                        self.slice_xz[quant].append(data_xz)
                        self.slice_yz[quant].append(data_yz)
                        self.slice_x[quant].append(data_x)
                        self.slice_y[quant].append(data_y)
                        self.slice_z[quant].append(data_z)

                    else:
                        # spec dependent
                        for spec in self.specnames:

                            data_in = getattr(getattr(data, file + '_' + spec), quant)(step.time, getattr(step, file))

                            if self.x_fourier:
                                if self.y_fourier:
                                    data_xyz = np.fft.fftshift(np.fft.fft2(data_in, axes=(0, 1))*np.conj(np.fft.fft(self.geom.jacobian.T, axes=(0, 1))), axes=(0, 1))/np.mean(self.geom.jacobian.T)/self.run_data.pnt.nx0**2/self.run_data.pnt.ny0**2
                                else:
                                    data_xyz = np.fft.fftshift(np.fft.fft(data_in, axis=0)*np.conj(np.fft.fft(self.geom.jacobian.T, axis=0)), axis=0)/np.mean(self.geom.jacobian.T)/self.run_data.pnt.nx0**2
                            else:
                                if self.y_fourier:
                                    data_xyz = np.fft.fftshift(np.fft.fft(data_in, axis=1)*np.conj(np.fft.fft(self.geom.jacobian.T, axis=1)), axis=1)/np.mean(self.geom.jacobian.T)/self.run_data.pnt.ny0**2
                                else:
                                    data_xyz = data_in*self.geom.jacobian.T/np.mean(self.geom.jacobian.T)

                            if self.opts['square']['value']: data_xyz = np.absolute(data_xyz)**2

                            data_xy = np.average(data_xyz[:, :, self.zind_min:self.zind_max+1], axis=2)
                            data_xz = np.average(data_xyz[:, self.yind_min:self.yind_max+1, :], axis=1)
                            data_yz = np.average(data_xyz[self.xind_min:self.xind_max+1, :, :], axis=0)

                            data_x = np.average(data_xyz[:, self.yind_min: self.yind_max+1, self.xind_min:self.xind_max+1], axis=(1, 2))
                            data_y = np.average(data_xyz[self.xind_min:self.xind_max+1, :, self.zind_min:self.zind_max+1], axis=(0, 2))
                            data_z = np.average(data_xyz[self.xind_min:self.xind_max+1, self.yind_min:self.yind_max+1, :], axis=(0, 1))

                            self.slice_xy[quant + '#' + spec].append(data_xy)
                            self.slice_xz[quant + '#' + spec].append(data_xz)
                            self.slice_yz[quant + '#' + spec].append(data_yz)
                            self.slice_x[quant + '#' + spec].append(data_x)
                            self.slice_y[quant + '#' + spec].append(data_y)
                            self.slice_z[quant + '#' + spec].append(data_z)


    def plot(self, time_requested, output=None, out_folder=None):

        if output:
            output.info_txt.insert(END, "Slice:\n")

        plotbase = Plotting()
        plotbase.titles.update(
                {"phi": r"$\phi$", "n": r"$n$", "u_par": r"u_{//}", "T_par": r"T_{//}",
                 "T_perp": r"T_{\perp}", "Q_es": r"$Q_{es}$", "Q_em": r"$Q_{em}$",
                 "Gamma_es": r"$\Gamma_{es}$", "Gamma_em": r"$\Gamma_{em}$"})

        def plot_a_map(ax, x, y, f, is_f, x_lbl, y_lbl, ttl):
            cm1 = ax.pcolormesh(x, y, np.abs(np.squeeze(f)).T if is_f else np.squeeze(f).T)
            ax.set_rasterization_zorder(z=-10)
            ax.set_xlabel(x_lbl)
            ax.set_ylabel(y_lbl)
            ax.set_title(ttl)
            cm1.cmap_bidirect

        def plot_a_quant(ax, x, data, is_complex, xlabel=None, title=None):
            ax.plot(x, np.absolute(data.T) if is_complex else data.T, color='b')
            ax.set_rasterization_zorder(z=-10)
            if xlabel:
                ax.set_xlabel(xlabel)
            if title:
                ax.set_title(title)

        if self.opts['f_x']['value']:
            x_lbl = r"$k_x\rho_{ref}$"
            x = np.fft.fftshift(self.run_data.spatialgrid.kx_fftorder)
            x_p = np.fft.fftshift(self.run_data.spatialgrid.kx_fftorder)

        else:
            x_lbl = r"$x/a$"
            x = self.run_data.spatialgrid.x_a
            x_p = x

        if self.opts['f_y']['value']:
            y_lbl = r"$k_y\rho_{ref}$"
            y = np.fft.fftshift(self.run_data.spatialgrid.ky)
            y_p = np.fft.fftshift(self.run_data.spatialgrid.ky)

        else:
            y_lbl = r"$y/\rho_{ref}$"
            y = self.run_data.spatialgrid.y
            y_p = y

        is_f = self.opts['f_x']['value'] or self.opts['f_y']['value'] or self.opts['abs']['value']


        z_lbl = r"$z/L_{ref}$"
        z = self.run_data.spatialgrid.z

        text_x = 'Avg. over x = [' + '{:.1f}'.format(self.run_data.spatialgrid.x_a[self.xind_min]) + ',' + '{:.1f}'.format(self.run_data.spatialgrid.x_a[self.xind_max-1])+ ']  '
        text_y = 'y = [' + '{:.1f}'.format(self.run_data.spatialgrid.y[self.yind_min]) + ',' + '{:.1f}'.format(self.run_data.spatialgrid.y[self.yind_max-1])+ ']  '
        text_z = 'z = [' + '{:.1f}'.format(self.run_data.spatialgrid.z[self.zind_min]) + ',' + '{:.1f}'.format(self.run_data.spatialgrid.z[self.zind_max-1])+ ']'
        text = text_x + text_y + text_z

        if self.opts['t_avg']['value']:
            # loop over file
            for quant in self.slice_xy:

                if self.opts['save_pdf']['value']:
                    pp2d = PdfPages(out_folder / 'slices2d_{}.pdf'.format(quant))
                    pp1d = PdfPages(out_folder / 'slices1d_{}.pdf'.format(quant))
                    plt.ioff()

                ind_str = quant.find('#')
                if ind_str == -1:
                    ttl = plotbase.titles[quant]
                else:
                    ttl = plotbase.titles[quant[0:ind_str]] + " " + quant[ind_str + 1:]

                fig = plt.figure(figsize=(15, 10))
                ax_1 = fig.add_subplot(1, 3, 1)
                plotbase.plot_contour_quant(ax_1, fig, x, y, averages.mytrapz(self.slice_xy[quant], time_requested), is_f, x_lbl, y_lbl, ttl)
                ax_2 = fig.add_subplot(1, 3, 2)
                plotbase.plot_contour_quant(ax_2, fig, x, z, averages.mytrapz(self.slice_xz[quant], time_requested), is_f, x_lbl, z_lbl, ttl)
                ax_3 = fig.add_subplot(1, 3, 3)
                plotbase.plot_contour_quant(ax_3, fig, y, z, averages.mytrapz(self.slice_yz[quant], time_requested), is_f, y_lbl, z_lbl, ttl)
                fig.suptitle(text)

                if self.opts['save_pdf']['value']:
                    fig.savefig(pp2d, format='pdf', bbox_inches='tight')
                    pp2d.close()
                else:
                    fig.show()

                fig = plt.figure(figsize=(15, 10))
                ax_1 = fig.add_subplot(1, 3, 1)
                plotbase.plot_1d_quant(ax_1, fig, x_p, averages.mytrapz(self.slice_x[quant], time_requested), is_f, x_lbl, ttl)
                ax_2 = fig.add_subplot(1, 3, 2)
                plotbase.plot_1d_quant(ax_2, fig, y_p, averages.mytrapz(self.slice_y[quant], time_requested), is_f, y_lbl, ttl)
                ax_3 = fig.add_subplot(1, 3, 3)
                plotbase.plot_1d_quant(ax_3, fig, z, averages.mytrapz(self.slice_z[quant], time_requested), is_f, z_lbl, ttl)
                fig.suptitle(text)

                if self.opts['save_pdf']['value']:
                    fig.savefig(pp1d, format='pdf', bbox_inches='tight')
                    pp1d.close()
                else:
                    fig.show()

                if self.opts['save_h5']['value']:
                    file = h5py.File(out_folder / 'slices_{}.h5'.format(quant), 'w')
                    file["/x"] = x
                    file["/y"] = y
                    file["/z"] = z
                    file["/" + quant + "_xy"] = averages.mytrapz(self.slice_xy[quant], time_requested)
                    file["/" + quant + "_xz"] = averages.mytrapz(self.slice_xz[quant], time_requested)
                    file["/" + quant + "_yz"] = averages.mytrapz(self.slice_yz[quant], time_requested)
                    file["/" + quant + "_x"] = averages.mytrapz(self.slice_x[quant], time_requested)
                    file["/" + quant + "_y"] = averages.mytrapz(self.slice_y[quant], time_requested)
                    file["/" + quant + "_z"] = averages.mytrapz(self.slice_z[quant], time_requested)
                    file.close()

        else:

            for quant in self.slice_xy:

                ind_str = quant.find('#')
                if ind_str == -1:
                    ttl = plotbase.titles[quant]
                else:
                    ttl = plotbase.titles[quant[0:ind_str]] + " " + quant[ind_str + 1:]

                if self.opts['save_pdf']['value']:

                    pp2d = PdfPages(out_folder / 'slices2d_{}.pdf'.format(quant))
                    pp1d = PdfPages(out_folder / 'slices1d_{}.pdf'.format(quant))
                    plt.ioff()
                    for i_t, tm in enumerate(time_requested):

                        fig = plt.figure(figsize=(15, 10))
                        ax_1 = fig.add_subplot(1, 3, 1)
                        plot_a_map(ax_1, x, y, self.slice_xy[quant][i_t], is_f, x_lbl, y_lbl, ttl + ', t = {:.2f}'.format(tm))
                        ax_2 = fig.add_subplot(1, 3, 2)
                        plot_a_map(ax_2, x, z, self.slice_xz[quant][i_t], is_f, x_lbl, z_lbl, ttl + ', t = {:.2f}'.format(tm))
                        ax_3 = fig.add_subplot(1, 3, 3)
                        plot_a_map(ax_3, y, z, self.slice_yz[quant][i_t], is_f, y_lbl, z_lbl, ttl + ', t = {:.2f}'.format(tm))
                        fig.suptitle(text)
                        fig.savefig(pp2d, format='pdf', bbox_inches='tight')

                        fig = plt.figure(figsize=(15, 10))
                        ax_1 = fig.add_subplot(1, 3, 1)
                        plot_a_quant(ax_1, x_p, self.slice_x[quant][i_t], is_f, x_lbl, ttl + ', t = {:.2f}'.format(tm))
                        ax_2 = fig.add_subplot(1, 3, 2)
                        plot_a_quant(ax_2, y_p, self.slice_y[quant][i_t], is_f, y_lbl, ttl + ', t = {:.2f}'.format(tm))
                        ax_3 = fig.add_subplot(1, 3, 3)
                        plot_a_quant(ax_3, z, self.slice_z[quant][i_t], is_f, z_lbl, ttl + ', t = {:.2f}'.format(tm))
                        fig.suptitle(text)
                        fig.savefig(pp1d, format='pdf', bbox_inches='tight')
                    pp2d.close()
                    pp1d.close()

                else:

                    fig = plt.figure(figsize=(15, 10))
                    ax_1 = fig.add_subplot(1, 3, 1)
                    plotbase.plot_contour_quant_timeseries(ax_1, fig, x, y, self.slice_xy[quant],
                                        is_f, time_requested, x_lbl, y_lbl, ttl, True)
                    ax_2 = fig.add_subplot(1, 3, 2)
                    plotbase.plot_contour_quant_timeseries(ax_2, fig, x, z, self.slice_xz[quant],
                                        is_f, time_requested, x_lbl, z_lbl, ttl, True)
                    ax_3 = fig.add_subplot(1, 3, 3)
                    plotbase.plot_contour_quant_timeseries(ax_3, fig, y, z, self.slice_yz[quant],
                                        is_f, time_requested, y_lbl, z_lbl, ttl, True)
                    fig.suptitle(text)
                    fig.show()


                    fig = plt.figure(figsize=(15, 10))
                    ax_1 = fig.add_subplot(1, 3, 1)
                    plotbase.plot_1d_quant_timeseries(ax_1, fig, x_p, self.slice_x[quant],
                                                      is_f, time_requested, x_lbl, ttl)
                    ax_2 = fig.add_subplot(1, 3, 2)
                    plotbase.plot_1d_quant_timeseries(ax_2, fig, y_p, self.slice_y[quant],
                                                      is_f, time_requested, y_lbl, ttl)
                    ax_3 = fig.add_subplot(1, 3, 3)
                    plotbase.plot_1d_quant_timeseries(ax_3, fig, z, self.slice_z[quant],
                                                      is_f, time_requested, z_lbl, ttl)
                    fig.suptitle(text)

                    fig.show()



# -*- coding: utf-8 -*-
""" Module to handle the output of a scan

including local scans derived from global profiles and scans producing neoclassical data
"""
import numpy as np

from ..utils.ParIO import Parameters
from ..utils.comm import CommonData
from .nrgdata import NrgFile


class Scandata:
    """ Collect the data of a scan and store them in nice numpy arrays"""
    def __init__(self):

        mainpars = Parameters()
        mainpars.Read_Pars("parameters")
        self.n_ev = mainpars.pardict.get("n_ev", 1)
        try:
            # The dimensions in the namelist are in inverse order of the scan.log file!
            self.scandims = [int(dim) for dim in mainpars.pardict["scan_dims"].split()][::-1]
        except AttributeError:
            # This occurs for only one scan dimension
            self.scandims = [mainpars.pardict["scan_dims"]]
        # Build a field for each scan parameter with the full scandimensions (concatenating tuples)
        self.grid = np.empty((tuple(self.scandims)+(len(self.scandims),)))
        # its also nice to have them as a list of 1d vectors
        self.redgrid = [np.empty(dim) for dim in self.scandims]
        self.growthrates = np.empty(self.scandims + [self.n_ev])
        self.frequencies = np.empty(self.scandims + [self.n_ev])
        self.neodata = {"Gammanc": np.empty(self.scandims), "Qnc": np.empty(self.scandims),
                        "Pinc": np.empty(self.scandims), "jbs": np.empty(self.scandims)}
        self.scannames = []     # Names of the parameters scanned
        self.fetchedpars = {}   # Contains the data fetched from individual parameter_xxxx files

    def parsescan(self):
        """ Parse scan.log for IV and NC and get additional data from parameters_run """
        try:
            with open("scan.log", 'r') as scanlog:
                header = scanlog.readline().split()
                self.set_scan_names(header)
                for line in scanlog:
                    lst = line.split()
                    # scan.log has column majority ("Fortran") order
                    coord = np.unravel_index(int(lst[0])-1, self.scandims, order='F')
                    for ipar in range(len(self.scandims)):
                        self.grid[coord + (ipar,)] = float(lst[2+2*ipar])
                    self.growthrates[coord] = [float(lst[-(2+3*i_ev)]) for i_ev in range(self.n_ev)]
                    self.frequencies[coord] = [float(lst[-(1+3*i_ev)]) for i_ev in range(self.n_ev)]
        except IOError:
            print("Could not read scan.log file")
            raise
        self._reducegrid()

    def _reducegrid(self):
        """ Reduce the big grid variable for the scan parameters to a list of arrays """
        if len(self.scandims) == 1:
            self.redgrid = {self.scannames[0]: self.grid}
        else:
            for idim, dimsize in enumerate(self.scandims):
                self.redgrid[idim] = np.rollaxis(self.grid[..., idim], idim).flatten(order='F')[
                                     0:dimsize]
            self.redgrid = {self.scannames[i]: self.redgrid[i] for i, _ in enumerate(self.scannames)}

    def parseneo(self):
        """ Parse neo.log from simulations with neoclassical diagnostics """
        self.parsescan()
        try:
            with open("neo.log", 'r') as neolog:
                neolog.readline()
                for line in neolog:
                    lst = line.split()
                    # neo.log has column majority ("Fortran") order
                    coord = self._scannum_to_coord(int(lst[0]))
                    self.neodata["Gammanc"][coord] = float(lst[-4])
                    self.neodata["Qnc"][coord] = float(lst[-3])
                    self.neodata["Pinc"][coord] = float(lst[-2])
                    self.neodata["jbs"][coord] = float(lst[-1])
        except IOError:
            print("Could not read neo.log file. Is this a neoclassical run?")
            raise

    def _scannum_to_coord(self, scannum):
        return np.unravel_index(scannum-1, self.scandims, order='F')

    def fetchpars(self, pars, reread=False):
        """ Get parameter values from the parameters_fext files

        Warning: This can take quite long depending on the file
        system and scan dimensions
        :param pars: List of the parameters to read
        :param reread: Force update of the already parsed parameters
        """
        if not pars:
            return
        if (not reread) and (self.fetchedpars.keys() == pars):
            return
        runpar = Parameters()
        self.fetchedpars = {parname: np.full(self.scandims, np.nan) for parname in pars}
        # The files are numbered from one to scandim.size
        for runnum in range(1, self.growthrates.size+1):
            fext = str(runnum).zfill(4)
            runpar.Read_Pars("parameters_"+fext)
            coord = self._scannum_to_coord(runnum)
            par_not_found = []
            for parkey in pars:
                try:
                    self.fetchedpars[parkey][coord] = runpar.pardict[parkey]
                except KeyError:
                    par_not_found.append(parkey)
            if par_not_found:
                print("Parameter(s) {} not found. Will return NaN for it.".format(par_not_found))
                for rp in par_not_found:
                    pars.remove(rp)
                par_not_found.clear()

    def set_scan_names(self, header):
        """ Extract scan parameter names

        :param header: First line of scan.log or neo.log files
        """
        self.scannames.clear()
        for ipar in range(len(self.scandims)):
            # First comes "#Run" then |, then the parameter, a spec number
            # another | until / followed by /Eigenvalue1
            self.scannames.append(header[2 + 3*ipar])

    def parse_nrgfiles_last_step(self):
        self.nrgscan = np.empty(self.scandims, dtype=np.ndarray)
        for runnum in range(1, self.growthrates.size+1):
            fext = "_{}".format(str(runnum).zfill(4))
            cm = CommonData(fext, -2, -2)
            nrg = NrgFile("nrg{}".format(fext), cm)
            nrg.generate_timeseries()
            coord = self._scannum_to_coord(runnum)
            self.nrgscan[coord] = nrg.dataarray



import numpy as np
from .interp import *

def get_mat_fd_d1_o4(size,dx,plot_matrix=False):
    """Creates matrix for centered finite difference, first derivative, 4th order.
    size: size of (number of elements in) quantity to be differentiated
    dx: grid spacing (for constant grid)."""

    prefactor=1.0/(12.0*dx)
    mat=np.zeros((size,size),dtype='float')    
    for i in range(size):
        if i-1 >= 0:
            mat[i,i-1]=-8
        if i-2 >= 0:
            mat[i,i-2]=1
        if i+1 <= size-1:
            mat[i,i+1]=8
        if i+2 <= size-1:
            mat[i,i+2]=-1
   
    mat=prefactor*mat

    if plot_matrix:
        plt.contourf(mat,50)
        plt.colorbar()
        plt.show()

    return mat

    
def fd_d1_o4(var,grid,mat=False):
    """Centered finite difference, first derivative, 4th order.
    var: quantity to be differentiated.
    grid: grid for var 
    mat: matrix for the finite-differencing operator. if mat=False then it is created"""

    if not mat:
        mat=get_mat_fd_d1_o4(len(var),grid[1]-grid[0])

    dvar=-np.dot(mat,var)
    dvar[0]=0.0
    dvar[1]=0.0
    #dvar[2]=0.0
    dvar[-1]=0.0
    dvar[-2]=0.0
    #dvar[-3]=0.0
    return -dvar 

def fd_d1_o4_uneven(var,grid,mat=False,return_new_grid = False):
    """Centered finite difference, first derivative, 4th order.  Evenly spaced grid is created and var is interpolated onto this grid.  Derivative is interpolated back onto original grid.
    var: quantity to be differentiated.
    grid: grid for var 
    mat: matrix for the finite-differencing operator. if mat=False then it is created"""

    N = 2.0*len(grid)
    grid0 = np.linspace(grid[0],grid[-1],N)
    var0 = interp(grid,var,grid0)

    if not mat:
        mat=get_mat_fd_d1_o4(len(var0),grid0[1]-grid0[0])

    dvar0=-np.dot(mat,var0)
    dvar0[0]=0.0
    dvar0[1]=0.0
    dvar0[-1]=0.0
    dvar0[-2]=0.0

    if return_new_grid:
        return grid0,-dvar0
    else:
        dvar = np.zeros(len(grid))
        dvar[2:-2] = interp(grid0[2:-2],dvar0[2:-2],grid[2:-2])
        return -dvar 

def fd_d1_o4_smoothend(var,grid,mat=False):
    """Centered finite difference, first derivative, 4th order using extrapolation to get boundary points
    var: quantity to be differentiated.
    grid: grid for var 
    mat: matrix for the finite-differencing operator. if mat=False then it is created"""

    dx = grid[1]-grid[0]
    grid0 = np.linspace(grid[0]-2*dx,grid[-1]+2*dx,len(grid)+4)
    var0 = interp(grid,var,grid0)

    if not mat:
        mat=get_mat_fd_d1_o4(len(var0),grid0[1]-grid0[0])

    dvar0=-np.dot(mat,var0)
    dvar_out=dvar0[2:-2]

    return -dvar_out 
 
    
def invert_fd_d1_o4(var,grid,mat=False):
    """Invert cenntered finite difference, first derivative, 4th order.
    var: quantity to be integrated.
    grid: grid for var 
    mat: matrix for the finite-differencing operator. if mat=False then it is created
    note--mat will be inverted for this operation"""

    if not mat:
        mat=get_mat_fd_d1_o4(len(var),grid[1]-grid[0])

    imat=np.linalg.inv(mat)

    ivar=np.dot(imat,var)
    return ivar 





# -*- coding: utf-8 -*-

import abc
from tkinter import Label, END, Frame, Text, Entry, OptionMenu, StringVar, BooleanVar
from tkinter import Tk, Toplevel
import itertools


class Diagnostic(abc.ABC):
    def __init__(self):
        """ When we initialize each diagnostic class this call the set buttons  """
        self.name = 'my_name'
        self.tab = 'my_tab'
        self.opts = {}
        self.needed_vars = {}

#    @abc.abstractmethod
#    def set_options(self, run_data, specnames=None):
#        """ This method must be called before starting the diagnstotic loop.
#            This Will set the optiopns for the specific run and return
#            info to the loader."""

    @abc.abstractmethod
    def execute(self, data, parameters, geometry, spatial_grid, step):
        """ The diagnostic itself"""

    @abc.abstractmethod
    def plot(self, time_requested, output=None, out_folder=None):
        """ Plot method plots the results"""

#    @abc.abstractmethod
#    def save(self, time_requested, output=None, out_folder=None):
#        """ This saves the data on disk"""


    def set_defaults(self):
        """ we set the default options, in case the option gui is not called"""
        for opt_id, opt_info in self.opts.items():
            #this fixes the quants and reloads them
            if "files" in self.opts[opt_id].keys():
                self.opts[opt_id]['values'] = self.list_avail_vars(self.opts[opt_id]['files'])
            # here we need to make sure that spec is set
            if opt_id == "spec":
                self.opts[opt_id]["values"] = self.list_specnames()
            if not opt_info['values']:
                # option has no default, e.g. might be different for local/global
                self.opts[opt_id]['values'] = None
                self.opts[opt_id]['value'] = None
                self.opts[opt_id]['bol'] = False
            elif isinstance(opt_info['values'][0], bool):
                # option is a boolean
                self.opts[opt_id]['value'] = self.opts[opt_id]['values'][0]
                self.opts[opt_id]['bol'] = True
            else:
                # option is not a boolean
                self.opts[opt_id]['value'] = self.opts[opt_id]['values'][0]
                self.opts[opt_id]['bol'] = False

    def list_avail_vars(self, filtered):
        """ filters the available data accoridng to request"""
        if self.avail_vars:
            return list(itertools.chain.from_iterable(
                [self.avail_vars[x].values()
                    for x in self.avail_vars.keys() if x in filtered]))
        else:
            return None

    def list_specnames(self):
        """ returns a list of all species plus all if n_spec>1"""
        if self.specnames:
            return ['all'] + self.specnames if len(self.specnames) > 1 else self.specnames
        else:
            return None

    def get_needed_vars(self, filtered=None):
        # if we give a list of file names, set corresponding entries to true
        if filtered:
            for file_key in self.avail_vars.keys():
                self.needed_vars[file_key] = {}
                for data_value in self.avail_vars[file_key].values():
                    self.needed_vars[file_key][data_value] = False
            for x in filtered:
                self.needed_vars[x] = {y: True for y in self.avail_vars[x].values()}
        else:
            # use this to loop over quants from options
            for file_key in self.avail_vars.keys():
                self.needed_vars[file_key] = {}
                for data_value in self.avail_vars[file_key].values():
                    if data_value in self.opts['quant']['value']:
                        if (file_key in self.opts['quant']['files']):
                            self.needed_vars[file_key][data_value] = True
                        else:
                            self.needed_vars[file_key][data_value] = False
                    else:
                        self.needed_vars[file_key][data_value] = False

    def get_spec_from_opts(self):
        spec_tmp = self.opts['spec']['value']
        self.specnames = [spec_tmp] if spec_tmp != 'all' else self.specnames

    def __reload__(self, avail_vars, specnames):
        """ when we select a fdiagnostic or we change the file keeping it selcted,
            we need to reload variables and species"""
        self.avail_vars = avail_vars
        self.specnames = specnames
        self.set_defaults()


    class OptionsGUI():
        """ This manages the additional gui to set options"""

        def __init__(self):
            """ Nothing to do"""

        def startup(self, diagnostic, gui):
            # TODO block gui allowing only one options at the time
            self = Toplevel()
            self.title(diagnostic.name + " options")
#            Tk.__init__(self)
#            Tk.wm_title(self, diagnostic.name + " options")
            print("Updating options for " + diagnostic.name)
            self.columnconfigure(0, weight=1)
            self.rowconfigure(0, weight=1)

            self.minsize(width=50, height=35)
            self.maxsize(width=self.winfo_screenheight(),
                         height=self.winfo_screenwidth())

            self.helpframe = Frame(self)
            self.helpframe.grid(row=0, column=0, sticky="nswe")
            self.help_txt = Text(self.helpframe)
            self.help_txt.grid(row=0, column=0, sticky="nswe",
                               padx=10, pady=10)
            self.help_txt.grid_columnconfigure(0, weight=1)
            self.help_txt.grid_rowconfigure(0, weight=1)
            self.help_txt.insert(END, diagnostic.help_txt)

            self.optframe = Frame(self)
            self.optframe.grid(row=0, column=1, sticky="nswe")

            self.helpframe.grid_columnconfigure(0, weight=1, uniform="group2")
            self.optframe.grid_columnconfigure(1, weight=1, uniform="group2")
            print(gui)
            # each time we creaate the gui we keep track of all options that
            #    the specific diagnostic has
            self.opts_input = []
            for i_opt, opt_name in enumerate(diagnostic.opts.keys()):
                self.opts_input.append(
                    Diagnostic.Option(self.optframe, opt_name, i_opt, diagnostic))
            print(gui)
            gui.wait_window(self)



    class Option:
        """ Here we crat the specific single option.
            Need to distinguish between paramters that can be set by the user
            or predefined ones"""

        def __init__(self, parent, opt_name, idx, diag):
            self.variable = StringVar(parent)
            self.label = Label(parent, text=diag.opts[opt_name]['tag'])
            self.label.grid(row=idx, column=0, sticky="nswe", padx=10, pady=10)

            if not diag.opts[opt_name]['values']:
                self.entry = Entry(parent, width=10, textvariable=self.variable)
                self.entry.grid(row=idx, column=1, sticky="nswe",
                                padx=10, pady=10)
                self.entry.bind('<Return>',
                                lambda event, diag=diag, opt_name=opt_name:
                                    self.opt_updt(event, diag, opt_name))
            else:
                self.variable.set(
                    str(diag.opts[opt_name]['value']) if diag.opts[opt_name]['bol'] else
                    diag.opts[opt_name]['value'])
                self.menu = OptionMenu(parent, self.variable,
                                       *[str(x) for x in diag.opts[opt_name]['values']] if
                                       diag.opts[opt_name]['bol'] else diag.opts[opt_name][
                                           'values'], command=lambda event, diag=diag,
                                                                     opt_name=opt_name:
                    self.opt_updt(event, diag, opt_name))
                self.menu.grid(row=idx, column=1, sticky="nswe",
                               padx=10, pady=10)

        def opt_updt(self, event=None, diag=None, opt_name=None):
            """ Updates the option value"""
            value = self.variable.get()
            if diag.opts[opt_name]['bol']:
                diag.opts[opt_name]['value'] = value == 'True'
            else:
                diag.opts[opt_name]['value'] = value
            print("Updated " + opt_name + " to ", diag.opts[opt_name]['value'])



""" Module containing averaging functions for diagnostics"""
import numpy as np
import os


def mytrapz(yvar, timefld):
    """ Trapezoid rule, adjusted for single timestep

    Operates on the first dimension of yvar (typically time)
    hence the name timefld for the integration variable samples
    :param yvar: Variable to integrate over
    :param timefld: The integration variable (time)
    :returns: The integration result (dim(yvar)-1)
    """
    timefld = np.array(timefld)
    yvar = np.real_if_close(yvar)
    if timefld.size == 1:
        return np.atleast_2d(yvar)[0]
    else:
        if yvar.shape[0] != len(timefld):
            raise ValueError("First dimension of yvar and timefld do not match")
        tmpt = timefld/(timefld[-1] - timefld[0])
        return np.trapz(yvar, x=tmpt, axis=0)


def z_av3d(var, geom):
    """ Perform the average in z direction for a 3d variable

    :param var: Variable to average over
    :param geom: GENE geomtry object

    """
    if geom.pnt.x_local:
        if geom.pnt.y_local:
            return np.average(var, weights=geom.jacobian, axis=-1)
        else:
            return np.average(var, weights=geom.jaco3d.T, axis=-1)
    else:
        if geom.pnt.y_local:
            return np.average(var, weights=geom.jaco3d.T, axis=-1)
        else:
            return np.average(var, weights=geom.jacobian.T, axis=2)


def y_av3d(var, geom):
    """ Perform the average in y direction for a 3d variable

    :param var: Variable to average over
    :param geom: GENE geomtry object
    """
    if geom.pnt.x_local:
        if geom.pnt.y_local:  # Add the negative half of the Fourier space for ky
            var[:, 0, :] *= 0.5
            return np.sum(2*var, axis=1)
        else:
            return np.average(var, weights=geom.jaco3d.T, axis=1)
    else:
        if geom.pnt.y_local:
            var[:, 0, :] *= 0.5
            return np.sum(2*var, axis=1)
        else:
            return np.average(var, weights=geom.jacobian.T, axis=1)


def x_av3d(var, geom):
    """ Perform the average in x direction for a 3d variable

    :param var: Variable to average over
    :param geom: GENE geomtry object
    """
    if geom.pnt.x_local:
        if geom.pnt.y_local:
            return np.sum(var, axis=0)
        else:  # Mirror positive kx to negative kx
            var[0, :, :] *= 0.5
            return np.sum(2*var, axis=0)
    else:
        if geom.pnt.y_local:
            return np.sum(var, axis=0)
        else:
            return np.average(var, weights=geom.jacobian.T, axis=0)


def xz_av3d(var, geom):
    """ Perform the average in x and z direction for a 3d variable

    :param var: Variable to average over
    :param geom: GENE geomtry object
    """
    if geom.pnt.x_local:
        if geom.pnt.y_local:
            return np.average(np.sum(var, axis=0), weights=geom.jacobian, axis=-1)
        else:  # Mirror positive kx to negative kx
            var[0, :, :] *= 0.5
            return np.average(np.sum(2*var, axis=0), weights=geom.jacobian.T, axis=-1)
    else:
        if geom.pnt.y_local:
            return np.average(var, weights=geom.jaco3d.T, axis=(0, -1))
        else:
            return np.average(var, weights=geom.jacobian.T, axis=(0, 2))


def yz_av3d(var, geom):
    """ Perform the average in y and z direction for a 3d variable
    param var: Variable to average over
    param geom: GENE geometry object """
    if geom.pnt.x_local:
        if geom.pnt.y_local:  # Add the negative half of the Fourier space for ky
            var[:, 0, :] *= 0.5
            return np.average(np.sum(2*var, axis=1), weights=geom.jacobian.T, axis=-1)
        else:
            return np.average(var, weights=geom.jaco3d.T, axis=(1, -1))  # geom has (y, z)
    else:
        if geom.pnt.y_local:
            return np.average(np.sum(2*var, axis=1) - var[:, 0, :], weights=geom.jacobian.T,
                              axis=-1)
        else:
            return np.average(var, weights=geom.jacobian.T, axis=(1, 2))


def xy_av3d_zprofile(var, geom):
    """ Perform the average in x and y direction for a 3d variable.  Includes 1/gxx**0.5 to match GENE diagnostic tool

    :param var: Variable to average over
    :param geom: GENE geomtry object
    """
    if geom.pnt.x_local:
        if geom.pnt.y_local:
            var[:, 0, :] *= 0.5
            return np.sum(2*var, axis=(0, 1))/geom.gxx**0.5
        else:  # Mirror positive kx to negative kx
            print("Error!  Only ready for local simulations.")
            os.exit()
    else:
        print("Error!  Only ready for local simulations.")
        os.exit()




def xy_av3d(var, geom):
    """ Perform the average in x and y direction for a 3d variable

    :param var: Variable to average over
    :param geom: GENE geomtry object
    """
    if geom.pnt.x_local:
        if geom.pnt.y_local:
            var[:, 0, :] *= 0.5
            return np.sum(2*var, axis=(0, 1))
        else:  # Mirror positive kx to negative kx
            var[0, :, :] *= 0.5
            return np.average(np.sum(2*var, axis=0), weights=geom.jacobian.T, axis=1)
    else:
        if geom.pnt.y_local:
            var[:, 0, :] *= 0.5
            return np.average(np.sum(2*var, axis=1), weights=geom.jacobian.T, axis=0)
        else:
            return np.average(var, weights=geom.jacobian.T, axis=(0, 1))


def xyz_av3d(var, geom):
    """ Perform the average in all spatial directions for a 3d variable

    :param var: Variable to average over
    :param geom: GENE geomtry object
    """
    if geom.pnt.x_local:
        if geom.pnt.y_local:
            var[:, 0, :] *= 0.5
            return np.average(np.sum(2*var, axis=(0, 1)), weights=geom.jacobian)
        else:  # Mirror positive kx to negative kx
            var[0, :, :] *= 0.5
            temp = np.sum(2*var, axis=0)
            return np.average(temp, weights=geom.jacobian.T, axis=(0, -1))
    else:
        if geom.pnt.y_local:
            var[:, 0, :] *= 0.5
            return np.average(np.sum(2*var, axis=1), weights=geom.jacobian.T, axis=(0, -1))
        else:
            return np.average(var, weights=geom.jacobian.T)


def av3d_by_switch(xavg, yavg, zavg):
    """ Map the required averaging function by binary switches for the averages

    :returns: Reference to the appropriate function from this module
    """
    # pylint: disable=too-many-return-statements
    if not (xavg or yavg or zavg):
        return lambda var, geom: var  # Return an identity function
    if xavg and not (yavg or zavg):
        return x_av3d
    if xavg and yavg and not zavg:
        return xy_av3d
    if xavg and not yavg and zavg:
        return xz_av3d
    if xavg and yavg and zavg:
        return xyz_av3d
    if not (xavg or yavg) and zavg:
        return z_av3d
    if not xavg and yavg and zavg:
        return yz_av3d
    if not xavg and yavg and not zavg:
        return y_av3d
    # Something must go very wrong to end here
    raise ValueError("Inconsistent averaging information")


def flux_spectra_xz_av(var, geom):
    """ Perform xz average in the form required for ky flux spectra (see diagplots.plot_spectra)"""
    if geom.pnt.x_local:
        if geom.pnt.y_local:
            temp = np.empty((geom.pnt.nky0, geom.pnt.nz0),dtype=np.float64)
            temp[0, :] = np.real(np.real(np.sum(var[:, 0, :], axis=0)))
            temp[1:, :] = 2*np.real(np.real(np.sum(var[:, 1:, :], axis=0)))
            
            return np.average(temp, weights=geom.jacobian, axis=-1)
        else:
            raise NotImplementedError("No support for y-global")
    else:
        raise NotImplementedError("No support for x-global yet")


def flux_spectra_yz_av(var, geom):
    """ Perform yz average in the form required for kx flux spectra (see diagplots.plot_spectra)"""
    if not geom.pnt.y_local:
        raise NotImplementedError("No support for y-global")
    if geom.pnt.x_local:
        nx0o2 = int(geom.pnt.nx0/2)
        temp = np.zeros((nx0o2 + 1, geom.pnt.nz0))
        temp[0, :] = np.real_if_close(var[0, 0, :]) + 2*np.sum(np.real(var[0, 1:, :]), axis=0)
        if geom.pnt.nx0%2 == 0:
            temp[1:nx0o2, :] = 2*np.real_if_close(var[1:nx0o2, 0, :]) + 2*np.sum(
                    np.real(var[1:nx0o2, 1:, :] + var[2*nx0o2:nx0o2:-1, 1:, :]), axis=1)
            temp[nx0o2, :] = np.real_if_close(var[nx0o2, 0, :]) + 2*np.sum(
                    np.real(var[nx0o2, 1:, :]), axis=0)
        else:
            temp[1:nx0o2 + 1, :] = 2*np.real_if_close(var[1:nx0o2 + 1, 0, :]) + 2*np.sum(
                    np.real(var[1:nx0o2 + 1, 1:, :] + var[2*nx0o2:nx0o2:-1, 1:, :]), axis=1)

        return np.average(temp, weights=geom.jacobian, axis=-1)
    else:
        raise NotImplementedError("No support for x-global yet")


def get_area(geom):
    """ Calculate flux surface area"""
    if geom.pnt.x_local:
        if geom.pnt.y_local:
            raise NotImplementedError("No support for x local yet")
        else:
            raise NotImplementedError("No support for y-global yet")
    else:
        if geom.pnt.y_local:
            raise NotImplementedError("No support for x-global yet")
        else:
            area_fs = np.zeros((geom.pnt.nx0))
            for z in range(0, geom.pnt.nz0):
                for y in range(0, geom.pnt.ny0):
                    area_fs = area_fs + np.sqrt(geom.gxx.T[:, y, z])*geom.jacobian.T[:, y, z]
            return area_fs*(2*np.pi) ** 2*abs(geom.Cy)/geom.pnt.ny0/geom.pnt.nz0

        
def flux_surface_avg(field, geometry):
    if geometry.jacobians.ndim==1:
       raise NotImplementedError("No support for x local yet") 
    elif geometry.jacobians.ndim==1:
        if field.dim==3:
            return np.average(np.sum(field,axis=1), weights=geom.jacobian, axis=-1)
        elif field.dim==2 and field.shape==geom.jacobian.shape:
            return np.average(field, weights=geom.jacobian, axis=-1)
        else:
            raise NotImplementedError("Operation not allowed")



from tkinter import END

import os
import h5py
import matplotlib.pyplot as plt
import numpy as np
import ..putils.averages as averages
import ..putils.fourier as fourier
from .baseplot import Plotting
from .diagnostic import Diagnostic

class DiagCorrelations(Diagnostic):
    # pylint: disable=invalid-name
    def __init__(self, avail_vars=None, specnames=None):
        super().__init__()
        self.name = 'Correlations'
        self.tabs = ['fluxtube']

        self.help_txt = """Makes correlation functions in x and y.\n
                        Saves z=-pi,+-pi/2, 0, and weighted average over those points.\n
                        \nSave pdf : save pdf file (def. False)
                        """

        self.avail_vars = avail_vars

        self.specnames = specnames

        self.opts = {"spec": {'tag': "Spec", 'values': self.list_specnames()}}

        self.set_defaults()
        self.save_meta_file = True
        self.save_meta_fname = 'correlations.h5'

        self.options_gui = Diagnostic.OptionsGUI()
        self.times = np.empty(0)

    def set_options(self, run, specnames, out_folder):
        self.has_EM = any(x.pnt.electromagnetic for x in run.parameters)
        self.get_needed_vars(['field', 'mom'])
        
        ##This is called in loader
        self.get_spec_from_opts()
        self.taux = np.empty((int(run.parameters[0].pardict['nx0'])))
        self.tauy = np.empty((int(2.0*run.parameters[0].pardict['nky0'])))
        
        if self.save_meta_file:
            self.setup_save_meta_h5(out_folder, run)
        self.corr_funcs = {}
        self.corr_lengths_x = {}
        self.corr_lengths_y = {}
        self.corr_funcs['phi']=np.zeros((run.parameters[0].pardict['nx0'],2*run.parameters[0].pardict['nky0'],5))
        self.corr_lengths_x['phi']=np.empty(5)
        self.corr_lengths_y['phi']=np.empty(5)
        if self.has_EM: 
            self.corr_funcs['A_par']=np.zeros((run.parameters[0].pardict['nx0'],2*run.parameters[0].pardict['nky0'],5))
            self.corr_lengths_x['A_par']=np.empty(5)
            self.corr_lengths_y['A_par']=np.empty(5)
        for spec in run.specnames:
            self.corr_funcs['dens_'+spec]=np.zeros((run.parameters[0].pardict['nx0'],2*run.parameters[0].pardict['nky0'],5))
            self.corr_lengths_x['dens_'+spec]=np.empty(5)
            self.corr_lengths_y['dens_'+spec]=np.empty(5)
            self.corr_funcs['u_par_'+spec]=np.zeros((run.parameters[0].pardict['nx0'],2*run.parameters[0].pardict['nky0'],5))
            self.corr_lengths_x['u_par_'+spec]=np.empty(5)
            self.corr_lengths_y['u_par_'+spec]=np.empty(5)
            self.corr_funcs['T_par_'+spec]=np.zeros((run.parameters[0].pardict['nx0'],2*run.parameters[0].pardict['nky0'],5))
            self.corr_lengths_x['T_par_'+spec]=np.empty(5)
            self.corr_lengths_y['T_par_'+spec]=np.empty(5)
            self.corr_funcs['T_perp_'+spec]=np.zeros((run.parameters[0].pardict['nx0'],2*run.parameters[0].pardict['nky0'],5))
            self.corr_lengths_x['T_perp_'+spec]=np.empty(5)
            self.corr_lengths_y['T_perp_'+spec]=np.empty(5)
        return self.needed_vars

    def execute(self, data, parameters, geometry, spatialgrid,  step):

        self.times =  np.append(self.times,step.time)
        xgrid,ygrid = fourier.xy_grids(data.field.phi(step.time,step.field,parameters),parameters)
        phi_xyz = fourier.fouriertransform_k2x_2D(data.field.phi(step.time,step.field,parameters),parameters)
        if self.has_EM:
            apar_xyz = fourier.fouriertransform_k2x_2D(data.field.A_par(step.time,step.field,parameters),parameters)
        dens_xyz = {}
        T_par_xyz = {}
        T_perp_xyz = {}
        u_par_xyz = {}
        for spec in self.specnames:
            dens = getattr(getattr(data, 'mom_' + spec), "dens")(step.time, step.mom, parameters)
            dens_xyz[spec] = fourier.fouriertransform_k2x_2D(dens,parameters)
            T_par = getattr(getattr(data, 'mom_' + spec), "T_par")(step.time, step.mom, parameters)
            T_par_xyz[spec] = fourier.fouriertransform_k2x_2D(T_par,parameters)
            T_perp = getattr(getattr(data, 'mom_' + spec), "T_perp")(step.time, step.mom, parameters)
            T_perp_xyz[spec] = fourier.fouriertransform_k2x_2D(T_perp,parameters)
            u_par = getattr(getattr(data, 'mom_' + spec), "u_par")(step.time, step.mom, parameters)
            u_par_xyz[spec] = fourier.fouriertransform_k2x_2D(u_par,parameters)
        #plt.contourf(ygrid,xgrid,abs(phi_xyz[:,:,int(parameters.pardict['nz0']/2.0)])) 
        #plt.show()
        #sample at z = -pi, -pi/2, 0, pi/2 correcting for edge_opt

        zindices = np.empty(4,dtype = 'int')
        zindices[0] = 0
        zindices[1] = translate_edge_opt(-np.pi/2.0,parameters.pardict['nz0'],parameters.pardict['edge_opt'])
        zindices[2] = translate_edge_opt(0.0,parameters.pardict['nz0'],parameters.pardict['edge_opt'])
        zindices[3] = translate_edge_opt(np.pi/2.0,parameters.pardict['nz0'],parameters.pardict['edge_opt'])

        for k in range(len(zindices)):
            print("k",k)
            self.taux,self.tauy,temp = corr_func_2D(phi_xyz[:,:,zindices[k]],phi_xyz[:,:,zindices[k]],xgrid,ygrid)
            self.corr_funcs['phi'][:,:,k] += temp
            self.corr_funcs['phi'][:,:,4] += np.sum(np.sum(abs(phi_xyz[:,:,zindices[k]]**2),axis=0),axis=0)*temp
            if self.has_EM:
                self.taux,self.tauy,self.corr_funcs['A_par'][:,:,k] = corr_func_2D(apar_xyz[:,:,zindices[k]],apar_xyz[:,:,zindices[k]],xgrid,ygrid)
                self.corr_funcs['A_par'][:,:,k] += temp
                self.corr_funcs['A_par'][:,:,4] += np.sum(np.sum(abs(apar_xyz[:,:,zindices[k]]**2),axis=0),axis=0)*temp

            specnum = 0
            for spec in self.specnames:

                self.taux,self.tauy,temp = corr_func_2D(dens_xyz[spec][:,:,zindices[k]],dens_xyz[spec][:,:,zindices[k]],xgrid,ygrid)
                self.corr_funcs['dens_'+spec][:,:,k] += temp
                self.corr_funcs['dens_'+spec][:,:,4] += np.sum(np.sum(abs(dens_xyz[spec][:,:,zindices[k]]**2),axis=0),axis=0)*temp

                self.taux,self.tauy,temp = corr_func_2D(T_par_xyz[spec][:,:,zindices[k]],T_par_xyz[spec][:,:,zindices[k]],xgrid,ygrid)
                self.corr_funcs['T_par_'+spec][:,:,k] += temp
                self.corr_funcs['T_par_'+spec][:,:,4] += np.sum(np.sum(abs(T_par_xyz[spec][:,:,zindices[k]]**2),axis=0),axis=0)*temp

                self.taux,self.tauy,temp = corr_func_2D(T_perp_xyz[spec][:,:,zindices[k]],T_perp_xyz[spec][:,:,zindices[k]],xgrid,ygrid)
                self.corr_funcs['T_perp_'+spec][:,:,k] += temp
                self.corr_funcs['T_perp_'+spec][:,:,4] += np.sum(np.sum(abs(T_perp_xyz[spec][:,:,zindices[k]]**2),axis=0),axis=0)*temp

                self.taux,self.tauy,temp = corr_func_2D(u_par_xyz[spec][:,:,zindices[k]],u_par_xyz[spec][:,:,zindices[k]],xgrid,ygrid)
                self.corr_funcs['u_par_'+spec][:,:,k] += temp
                self.corr_funcs['u_par_'+spec][:,:,4] += np.sum(np.sum(abs(u_par_xyz[spec][:,:,zindices[k]]**2),axis=0),axis=0)*temp
            specnum += 1
           
        #if self.save_meta_file:
        #    self.save_meta_h5(step.time)
    def calc_corr_length(self,tau,cfunc):
        max_corr = max(np.abs(cfunc))
        corr_len = 0.0
        i = 0
        while corr_len==0.0:
            if (abs(cfunc[i])-max_corr/np.e) > 0.0 and \
               (abs(cfunc[i+1])-max_corr/np.e) <= 0.0:
                slope=(cfunc[i+1]-cfunc[i])/(tau[i+1]-tau[i])
                zero=cfunc[i]-slope*tau[i]
                corr_len=(max_corr/np.e-zero)/slope
            i+=1
        return corr_len

    def finalize(self):

        for k in range(5):
            self.corr_lengths_x['phi'][k] = self.calc_corr_length(self.taux,np.sum(self.corr_funcs['phi'][:,:,k],axis=1))
            self.corr_lengths_y['phi'][k] = self.calc_corr_length(self.tauy,np.sum(self.corr_funcs['phi'][:,:,k],axis=0))
            if self.has_EM:
                self.corr_lengths_x['A_par'][k] = self.calc_corr_length(self.taux,np.sum(self.corr_funcs['A_par'][:,:,k],axis=1))
                self.corr_lengths_y['A_par'][k] = self.calc_corr_length(self.tauy,np.sum(self.corr_funcs['A_par'][:,:,k],axis=0))
            for spec in self.specnames:
                self.corr_lengths_x['dens_'+spec][k] = self.calc_corr_length(self.taux,np.sum(self.corr_funcs['dens_'+spec][:,:,k],axis=1))
                self.corr_lengths_y['dens_'+spec][k] = self.calc_corr_length(self.tauy,np.sum(self.corr_funcs['dens_'+spec][:,:,k],axis=0))
                self.corr_lengths_x['T_par_'+spec][k] = self.calc_corr_length(self.taux,np.sum(self.corr_funcs['T_par_'+spec][:,:,k],axis=1))
                self.corr_lengths_y['T_par_'+spec][k] = self.calc_corr_length(self.tauy,np.sum(self.corr_funcs['T_par_'+spec][:,:,k],axis=0))
                self.corr_lengths_x['u_par_'+spec][k] = self.calc_corr_length(self.taux,np.sum(self.corr_funcs['u_par_'+spec][:,:,k],axis=1))
                self.corr_lengths_y['u_par_'+spec][k] = self.calc_corr_length(self.tauy,np.sum(self.corr_funcs['u_par_'+spec][:,:,k],axis=0))
                self.corr_lengths_x['T_perp_'+spec][k] = self.calc_corr_length(self.taux,np.sum(self.corr_funcs['T_perp_'+spec][:,:,k],axis=1))
                self.corr_lengths_y['T_perp_'+spec][k] = self.calc_corr_length(self.tauy,np.sum(self.corr_funcs['T_perp_'+spec][:,:,k],axis=0))
        if self.save_meta_file:
            self.save_meta_h5()
                
    def setup_save_meta_h5(self, folder, run):
        if not os.path.isdir(folder):
            raise Exception(os.path.abspath(folder) + " does not exist or is undefined")

        nx = int(run.parameters[0].pardict['nx0'])
        ny = int(run.parameters[0].pardict['nky0']*2)
        self.h5_meta = os.path.join(folder, self.save_meta_fname)
        ##self.nkx_2 = len(run.spatialgrid[0].kx_pos)
        ##self.nky = len(run.spatialgrid[0].ky)
        if os.path.isfile(self.h5_meta):
            os.remove(self.h5_meta)
        if os.path.isfile(self.h5_meta):    
            # file exist, simply open it
            self.h5_meta = h5py.File(self.h5_meta, 'a')
        else:
            # file does not exit 
            self.h5_meta = h5py.File(self.h5_meta, 'w')

    def save_meta_h5(self ):
        #self.meta_file_pos += 1
        self.h5_meta.create_dataset('deltax',data = self.taux)
        self.h5_meta.create_dataset('deltay',data = self.tauy)
        self.h5_meta.create_dataset('times',data = self.times)
        self.h5_meta.create_dataset('Corr func phi',data = self.corr_funcs['phi'])
        self.h5_meta.create_dataset('lambda_x phi',data = self.corr_lengths_x['phi'])
        self.h5_meta.create_dataset('lambda_y phi',data = self.corr_lengths_y['phi'])
        if self.has_EM:
            self.h5_meta.create_dataset('Corr func A_par',data = self.corr_funcs['A_par'])
            self.h5_meta.create_dataset('lambda_x A_par',data = self.corr_lengths_x['A_par'])
            self.h5_meta.create_dataset('lambda_y A_par',data = self.corr_lengths_y['A_par'])
        for spec in self.specnames:
            self.h5_meta.create_dataset('Corr func dens_'+spec,data = self.corr_funcs['dens_'+spec])
            self.h5_meta.create_dataset('lambda_x dens_'+spec,data = self.corr_lengths_x['dens_'+spec])
            self.h5_meta.create_dataset('lambda_y dens_'+spec,data = self.corr_lengths_y['dens_'+spec])
            self.h5_meta.create_dataset('Corr func T_par_'+spec,data = self.corr_funcs['T_par_'+spec])
            self.h5_meta.create_dataset('lambda_x T_par_'+spec,data = self.corr_lengths_x['T_par_'+spec])
            self.h5_meta.create_dataset('lambda_y T_par_'+spec,data = self.corr_lengths_y['T_par_'+spec])
            self.h5_meta.create_dataset('Corr func T_perp_'+spec,data = self.corr_funcs['T_perp_'+spec])
            self.h5_meta.create_dataset('lambda_x T_perp_'+spec,data = self.corr_lengths_x['T_perp_'+spec])
            self.h5_meta.create_dataset('lambda_y T_perp_'+spec,data = self.corr_lengths_y['T_perp_'+spec])
            self.h5_meta.create_dataset('Corr func u_par_'+spec,data = self.corr_funcs['u_par_'+spec])
            self.h5_meta.create_dataset('lambda_x u_par_'+spec,data = self.corr_lengths_x['u_par_'+spec])
            self.h5_meta.create_dataset('lambda_y u_par_'+spec,data = self.corr_lengths_y['u_par_'+spec])
                #self.h5_meta['/time'].resize((self.meta_file_pos), axis=0)
        #self.h5_meta['/time'][-1] = time
        self.h5_meta.flush()
        
    def close_meta_h5(self):
        self.h5_meta.close()

    def load_meta_h5(self, out_folder):
        self.h5_meta = os.path.join(out_folder, self.save_meta_fname)
        self.h5_meta = h5py.File(self.h5_meta, 'a')
        list(self.h5_meta.keys())
        
        self.corr_lengths_x['phi']=self.h5_meta['lambda_x phi'][:]
        self.corr_lengths_y['phi']=self.h5_meta['lambda_y phi'][:]
        self.corr_funcs['phi']=self.h5_meta['Corr func phi'][:,:,:]
        if self.has_EM:
            self.corr_lengths_x['A_par']=self.h5_meta['lambda_x A_par'][:]
            self.corr_lengths_y['A_par']=self.h5_meta['lambda_y A_par'][:]
            self.corr_funcs['A_par']=self.h5_meta['Corr func A_par'][:,:,:]
        for spec in self.specnames:
            self.corr_lengths_x['dens_'+spec]=self.h5_meta['lambda_x dens_'+spec][:]
            self.corr_lengths_y['dens_'+spec]=self.h5_meta['lambda_y dens_'+spec][:]
            self.corr_funcs['dens_'+spec]=self.h5_meta['Corr func dens_'+spec][:,:,:]
            self.corr_lengths_x['u_par_'+spec]=self.h5_meta['lambda_x u_par_'+spec][:]
            self.corr_lengths_y['u_par_'+spec]=self.h5_meta['lambda_y u_par_'+spec][:]
            self.corr_funcs['u_par_'+spec]=self.h5_meta['Corr func u_par_'+spec][:,:,:]
            self.corr_lengths_x['T_perp_'+spec]=self.h5_meta['lambda_x T_perp_'+spec][:]
            self.corr_lengths_y['T_perp_'+spec]=self.h5_meta['lambda_y T_perp_'+spec][:]
            self.corr_funcs['T_perp_'+spec]=self.h5_meta['Corr func T_perp_'+spec][:,:,:]
            self.corr_lengths_x['T_par_'+spec]=self.h5_meta['lambda_x T_par_'+spec][:]
            self.corr_lengths_y['T_par_'+spec]=self.h5_meta['lambda_y T_par_'+spec][:]
            self.corr_funcs['T_par_'+spec]=self.h5_meta['Corr func T_par_'+spec][:,:,:]
        self.h5_meta.close()
        
    def plot(self, time_requested, output=None, out_folder=None, terminal=None):
        """ 
            Plot."""

        self.load_meta_h5(out_folder)
        #plt.figure(figsize = (18,9))
        plt.figure(figsize = (18,8))
        zind = 0
        plt.subplot(3,5,1)
        plt.title(r'$|\phi|^2(\Delta x, \Delta y)(z=-\pi)$')
        plt.contourf(self.tauy,self.taux,self.corr_funcs['phi'][:,:,zind])
        plt.colorbar()
        plt.ylabel(r'$\Delta x(\rho_s)$')
        plt.xlabel(r'$\Delta y(\rho_s)$')
        plt.subplot(3,5,6)
        plt.plot(self.taux,np.sum(self.corr_funcs['phi'][:,:,zind],axis=1))
        plt.axvline(self.corr_lengths_x['phi'][zind],color = 'black',label=str(self.corr_lengths_x['phi'][zind])[:6])
        plt.xlabel(r'$\Delta x(\rho_s)$')
        plt.ylabel(r'$|\phi|^2(\Delta x)(z=-\pi)$')
        plt.legend()
        plt.subplot(3,5,11)
        plt.plot(self.tauy,np.sum(self.corr_funcs['phi'][:,:,zind],axis=0))
        plt.axvline(self.corr_lengths_y['phi'][zind],color = 'black',label=str(self.corr_lengths_y['phi'][zind])[:6])
        plt.xlabel(r'$\Delta y(\rho_s)$')
        plt.ylabel(r'$|\phi|^2(\Delta y)(z=-\pi)$')
        plt.legend()

        zind = 1
        plt.subplot(3,5,2)
        plt.title(r'$|\phi|^2(\Delta x, \Delta y)(z=-\pi/2)$')
        plt.contourf(self.tauy,self.taux,self.corr_funcs['phi'][:,:,zind])
        plt.colorbar()
        plt.ylabel(r'$\Delta x(\rho_s)$')
        plt.xlabel(r'$\Delta y(\rho_s)$')
        plt.subplot(3,5,7)
        plt.plot(self.taux,np.sum(self.corr_funcs['phi'][:,:,zind],axis=1))
        plt.axvline(self.corr_lengths_x['phi'][zind],color = 'black',label=str(self.corr_lengths_x['phi'][zind])[:6])
        plt.xlabel(r'$\Delta x(\rho_s)$')
        plt.ylabel(r'$|\phi|^2(\Delta x)(z=-\pi/2)$')
        plt.legend()
        plt.subplot(3,5,12)
        plt.plot(self.tauy,np.sum(self.corr_funcs['phi'][:,:,zind],axis=0))
        plt.axvline(self.corr_lengths_y['phi'][zind],color = 'black',label=str(self.corr_lengths_y['phi'][zind])[:6])
        plt.xlabel(r'$\Delta y(\rho_s)$')
        plt.ylabel(r'$|\phi|^2(\Delta y)(z=-\pi/2)$')
        plt.legend()

        zind = 2
        plt.subplot(3,5,3)
        plt.title(r'$|\phi|^2(\Delta x, \Delta y)(z=0)$')
        plt.contourf(self.tauy,self.taux,self.corr_funcs['phi'][:,:,zind])
        plt.colorbar()
        plt.ylabel(r'$\Delta x(\rho_s)$')
        plt.xlabel(r'$\Delta y(\rho_s)$')
        plt.subplot(3,5,8)
        plt.plot(self.taux,np.sum(self.corr_funcs['phi'][:,:,zind],axis=1))
        plt.axvline(self.corr_lengths_x['phi'][zind],color = 'black',label=str(self.corr_lengths_x['phi'][zind])[:6])
        plt.xlabel(r'$\Delta x(\rho_s)$')
        plt.ylabel(r'$|\phi|^2(\Delta x)(z=0)$')
        plt.legend()
        plt.subplot(3,5,13)
        plt.plot(self.tauy,np.sum(self.corr_funcs['phi'][:,:,zind],axis=0))
        plt.axvline(self.corr_lengths_y['phi'][zind],color = 'black',label=str(self.corr_lengths_y['phi'][zind])[:6])
        plt.xlabel(r'$\Delta y(\rho_s)$')
        plt.ylabel(r'$|\phi|^2(\Delta y)(z=0)$')
        plt.legend()

        zind = 3
        plt.subplot(3,5,4)
        plt.title(r'$|\phi|^2(\Delta x, \Delta y)(z=\pi/2)$')
        plt.contourf(self.tauy,self.taux,self.corr_funcs['phi'][:,:,zind])
        plt.colorbar()
        plt.ylabel(r'$\Delta x(\rho_s)$')
        plt.xlabel(r'$\Delta y(\rho_s)$')
        plt.subplot(3,5,9)
        plt.plot(self.taux,np.sum(self.corr_funcs['phi'][:,:,zind],axis=1))
        plt.axvline(self.corr_lengths_x['phi'][zind],color = 'black',label=str(self.corr_lengths_x['phi'][zind])[:6])
        plt.xlabel(r'$\Delta x(\rho_s)$')
        plt.ylabel(r'$|\phi|^2(\Delta x)(z=\pi/2)$')
        plt.legend()
        plt.subplot(3,5,14)
        plt.plot(self.tauy,np.sum(self.corr_funcs['phi'][:,:,zind],axis=0))
        plt.axvline(self.corr_lengths_y['phi'][zind],color = 'black',label=str(self.corr_lengths_y['phi'][zind])[:6])
        plt.xlabel(r'$\Delta y(\rho_s)$')
        plt.ylabel(r'$|\phi|^2(\Delta y)(z=\pi/2)$')
        plt.legend()

        zind = 4
        plt.subplot(3,5,5)
        plt.title(r'$|\phi|^2(\Delta x, \Delta y)(z=avg)$')
        plt.contourf(self.tauy,self.taux,self.corr_funcs['phi'][:,:,zind])
        plt.colorbar()
        plt.ylabel(r'$\Delta x(\rho_s)$')
        plt.xlabel(r'$\Delta y(\rho_s)$')
        plt.subplot(3,5,10)
        plt.plot(self.taux,np.sum(self.corr_funcs['phi'][:,:,zind],axis=1))
        plt.axvline(self.corr_lengths_x['phi'][zind],color = 'black',label=str(self.corr_lengths_x['phi'][zind])[:6])
        plt.xlabel(r'$\Delta x(\rho_s)$')
        plt.ylabel(r'$|\phi|^2(\Delta x)(z=avg)$')
        plt.legend()
        plt.subplot(3,5,15)
        plt.plot(self.tauy,np.sum(self.corr_funcs['phi'][:,:,zind],axis=0))
        plt.axvline(self.corr_lengths_y['phi'][zind],color = 'black',label=str(self.corr_lengths_y['phi'][zind])[:6])
        plt.xlabel(r'$\Delta y(\rho_s)$')
        plt.ylabel(r'$|\phi|^2(\Delta y)(z=avg)$')
        plt.legend()
        plt.tight_layout()
        plt.show()

        if self.has_EM:
            plt.figure(figsize = (18,8))
            zind = 0
            plt.subplot(3,5,1)
            plt.title(r'$|A_{||}|^2(\Delta x, \Delta y)(z=-\pi)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['A_par'][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,6)
            plt.plot(self.taux,np.sum(self.corr_funcs['A_par'][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['A_par'][zind],color = 'black',label=str(self.corr_lengths_x['A_par'][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(r'$|A_{||}^2(\Delta x)(z=-\pi)$')
            plt.legend()
            plt.subplot(3,5,11)
            plt.plot(self.tauy,np.sum(self.corr_funcs['A_par'][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['A_par'][zind],color = 'black',label=str(self.corr_lengths_y['A_par'][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(r'$|A_{||}|^2(\Delta y)(z=-\pi)$')
            plt.legend()
    
            zind = 1
            plt.subplot(3,5,2)
            plt.title(r'$|A_{||}|^2(\Delta x, \Delta y)(z=-\pi/2)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['A_par'][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,7)
            plt.plot(self.taux,np.sum(self.corr_funcs['A_par'][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['A_par'][zind],color = 'black',label=str(self.corr_lengths_x['A_par'][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(r'$|A_{||}|^2(\Delta x)(z=-\pi/2)$')
            plt.legend()
            plt.subplot(3,5,12)
            plt.plot(self.tauy,np.sum(self.corr_funcs['A_par'][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['A_par'][zind],color = 'black',label=str(self.corr_lengths_y['A_par'][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(r'$|A_{||}|^2(\Delta y)(z=-\pi/2)$')
            plt.legend()
    
            zind = 2
            plt.subplot(3,5,3)
            plt.title(r'$|A_{||}|^2(\Delta x, \Delta y)(z=0)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['A_par'][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,8)
            plt.plot(self.taux,np.sum(self.corr_funcs['A_par'][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['A_par'][zind],color = 'black',label=str(self.corr_lengths_x['A_par'][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(r'$|A_{||}|^2(\Delta x)(z=0)$')
            plt.legend()
            plt.subplot(3,5,13)
            plt.plot(self.tauy,np.sum(self.corr_funcs['A_par'][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['A_par'][zind],color = 'black',label=str(self.corr_lengths_y['A_par'][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(r'$|A_{||}|^2(\Delta y)(z=0)$')
            plt.legend()
    
            zind = 3
            plt.subplot(3,5,4)
            plt.title(r'$|A_{||}|^2(\Delta x, \Delta y)(z=\pi/2)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['A_par'][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,9)
            plt.plot(self.taux,np.sum(self.corr_funcs['A_par'][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['A_par'][zind],color = 'black',label=str(self.corr_lengths_x['A_par'][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(r'$|A_{||}|^2(\Delta x)(z=\pi/2)$')
            plt.legend()
            plt.subplot(3,5,14)
            plt.plot(self.tauy,np.sum(self.corr_funcs['A_par'][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['A_par'][zind],color = 'black',label=str(self.corr_lengths_y['A_par'][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(r'$|A_{||}|^2(\Delta y)(z=\pi/2)$')
            plt.legend()
    
            zind = 4
            plt.subplot(3,5,5)
            plt.title(r'$|A_{||}|^2(\Delta x, \Delta y)(z=avg)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['A_par'][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,10)
            plt.plot(self.taux,np.sum(self.corr_funcs['A_par'][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['A_par'][zind],color = 'black',label=str(self.corr_lengths_x['A_par'][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(r'$|A_{||}|^2(\Delta x)(z=avg)$')
            plt.legend()
            plt.subplot(3,5,15)
            plt.plot(self.tauy,np.sum(self.corr_funcs['A_par'][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['A_par'][zind],color = 'black',label=str(self.corr_lengths_y['A_par'][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(r'$|A_{||}|^2(\Delta y)(z=avg)$')
            plt.legend()
            plt.tight_layout()
            plt.show()
            
        for spec in self.specnames:
            #density
            plt.figure(figsize = (18,8))
            zind = 0
            plt.subplot(3,5,1)
            plt.title(spec+' '+r'$|n|^2(\Delta x, \Delta y)(z=-\pi)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['dens_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,6)
            plt.plot(self.taux,np.sum(self.corr_funcs['dens_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['dens_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['dens_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|n^2(\Delta x)(z=-\pi)$')
            plt.legend()
            plt.subplot(3,5,11)
            plt.plot(self.tauy,np.sum(self.corr_funcs['dens_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['dens_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['dens_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|n|^2(\Delta y)(z=-\pi)$')
            plt.legend()
    
            zind = 1
            plt.subplot(3,5,2)
            plt.title(spec+' '+r'$|n|^2(\Delta x, \Delta y)(z=-\pi/2)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['dens_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,7)
            plt.plot(self.taux,np.sum(self.corr_funcs['dens_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['dens_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['dens_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|n|^2(\Delta x)(z=-\pi/2)$')
            plt.legend()
            plt.subplot(3,5,12)
            plt.plot(self.tauy,np.sum(self.corr_funcs['dens_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['dens_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['dens_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|n|^2(\Delta y)(z=-\pi/2)$')
            plt.legend()
    
            zind = 2
            plt.subplot(3,5,3)
            plt.title(spec+' '+r'$|n|^2(\Delta x, \Delta y)(z=0)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['dens_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,8)
            plt.plot(self.taux,np.sum(self.corr_funcs['dens_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['dens_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['dens_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|n|^2(\Delta x)(z=0)$')
            plt.legend()
            plt.subplot(3,5,13)
            plt.plot(self.tauy,np.sum(self.corr_funcs['dens_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['dens_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['dens_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|n|^2(\Delta y)(z=0)$')
            plt.legend()
    
            zind = 3
            plt.subplot(3,5,4)
            plt.title(spec+' '+r'$|n|^2(\Delta x, \Delta y)(z=\pi/2)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['dens_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,9)
            plt.plot(self.taux,np.sum(self.corr_funcs['dens_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['dens_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['dens_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|n|^2(\Delta x)(z=\pi/2)$')
            plt.legend()
            plt.subplot(3,5,14)
            plt.plot(self.tauy,np.sum(self.corr_funcs['dens_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['dens_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['dens_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|n|^2(\Delta y)(z=\pi/2)$')
            plt.legend()
    
            zind = 4
            plt.subplot(3,5,5)
            plt.title(spec+' '+r'$|n|^2(\Delta x, \Delta y)(avg)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['dens_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,10)
            plt.plot(self.taux,np.sum(self.corr_funcs['dens_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['dens_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['dens_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|n|^2(\Delta x)(z=avg)$')
            plt.legend()
            plt.subplot(3,5,15)
            plt.plot(self.tauy,np.sum(self.corr_funcs['dens_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['dens_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['dens_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|n|^2(\Delta y)(z=avg)$')
            plt.legend()
            plt.tight_layout()
            plt.show()

            #T_par
            plt.figure(figsize = (18,8))
            zind = 0
            plt.subplot(3,5,1)
            plt.title(spec+' '+r'$|T_{||}|^2(\Delta x, \Delta y)(z=-\pi)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['T_par_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,6)
            plt.plot(self.taux,np.sum(self.corr_funcs['T_par_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['T_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['T_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{||}^2(\Delta x)(z=-\pi)$')
            plt.legend()
            plt.subplot(3,5,11)
            plt.plot(self.tauy,np.sum(self.corr_funcs['T_par_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['T_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['T_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{||}|^2(\Delta y)(z=-\pi)$')
            plt.legend()
    
            zind = 1
            plt.subplot(3,5,2)
            plt.title(spec+' '+r'$|T_{||}|^2(\Delta x, \Delta y)(z=-\pi/2)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['T_par_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,7)
            plt.plot(self.taux,np.sum(self.corr_funcs['T_par_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['T_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['T_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{||}|^2(\Delta x)(z=-\pi/2)$')
            plt.legend()
            plt.subplot(3,5,12)
            plt.plot(self.tauy,np.sum(self.corr_funcs['T_par_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['T_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['T_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{||}|^2(\Delta y)(z=-\pi/2)$')
            plt.legend()
    
            zind = 2
            plt.subplot(3,5,3)
            plt.title(spec+' '+r'$|T_{||}|^2(\Delta x, \Delta y)(z=0)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['T_par_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,8)
            plt.plot(self.taux,np.sum(self.corr_funcs['T_par_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['T_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['T_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{||}|^2(\Delta x)(z=0)$')
            plt.legend()
            plt.subplot(3,5,13)
            plt.plot(self.tauy,np.sum(self.corr_funcs['T_par_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['T_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['T_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{||}|^2(\Delta y)(z=0)$')
            plt.legend()
    
            zind = 3
            plt.subplot(3,5,4)
            plt.title(spec+' '+r'$|T_{||}|^2(\Delta x, \Delta y)(z=\pi/2)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['T_par_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,9)
            plt.plot(self.taux,np.sum(self.corr_funcs['T_par_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['T_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['T_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{||}|^2(\Delta x)(z=\pi/2)$')
            plt.legend()
            plt.subplot(3,5,14)
            plt.plot(self.tauy,np.sum(self.corr_funcs['T_par_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['T_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['T_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{||}|^2(\Delta y)(z=\pi/2)$')
            plt.legend()
    
            zind = 4
            plt.subplot(3,5,5)
            plt.title(spec+' '+r'$|T_{||}|^2(\Delta x, \Delta y)(avg)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['T_par_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,10)
            plt.plot(self.taux,np.sum(self.corr_funcs['T_par_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['T_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['T_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{||}|^2(\Delta x)(z=avg)$')
            plt.legend()
            plt.subplot(3,5,15)
            plt.plot(self.tauy,np.sum(self.corr_funcs['T_par_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['T_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['T_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{||}|^2(\Delta y)(z=avg)$')
            plt.legend()
            plt.tight_layout()
            plt.show()

            #T_perp
            plt.figure(figsize = (18,8))
            zind = 0
            plt.subplot(3,5,1)
            plt.title(spec+' '+r'$|T_{\perp}|^2(\Delta x, \Delta y)(z=-\pi)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['T_perp_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,6)
            plt.plot(self.taux,np.sum(self.corr_funcs['T_perp_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['T_perp_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['T_perp_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{\perp}^2(\Delta x)(z=-\pi)$')
            plt.legend()
            plt.subplot(3,5,11)
            plt.plot(self.tauy,np.sum(self.corr_funcs['T_perp_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['T_perp_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['T_perp_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{/perp}|^2(\Delta y)(z=-\pi)$')
            plt.legend()
    
            zind = 1
            plt.subplot(3,5,2)
            plt.title(spec+' '+r'$|T_{\perp}|^2(\Delta x, \Delta y)(z=-\pi/2)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['T_perp_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,7)
            plt.plot(self.taux,np.sum(self.corr_funcs['T_perp_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['T_perp_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['T_perp_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{/perp}|^2(\Delta x)(z=-\pi/2)$')
            plt.legend()
            plt.subplot(3,5,12)
            plt.plot(self.tauy,np.sum(self.corr_funcs['T_perp_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['T_perp_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['T_perp_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{/perp}|^2(\Delta y)(z=-\pi/2)$')
            plt.legend()
    
            zind = 2
            plt.subplot(3,5,3)
            plt.title(spec+' '+r'$|T_{\perp}|^2(\Delta x, \Delta y)(z=0)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['T_perp_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,8)
            plt.plot(self.taux,np.sum(self.corr_funcs['T_perp_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['T_perp_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['T_perp_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{/perp}|^2(\Delta x)(z=0)$')
            plt.legend()
            plt.subplot(3,5,13)
            plt.plot(self.tauy,np.sum(self.corr_funcs['T_perp_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['T_perp_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['T_perp_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{/perp}|^2(\Delta y)(z=0)$')
            plt.legend()
    
            zind = 3
            plt.subplot(3,5,4)
            plt.title(spec+' '+r'$|T_{\perp}|^2(\Delta x, \Delta y)(z=\pi/2)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['T_perp_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,9)
            plt.plot(self.taux,np.sum(self.corr_funcs['T_perp_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['T_perp_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['T_perp_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{/perp}|^2(\Delta x)(z=\pi/2)$')
            plt.legend()
            plt.subplot(3,5,14)
            plt.plot(self.tauy,np.sum(self.corr_funcs['T_perp_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['T_perp_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['T_perp_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{/perp}|^2(\Delta y)(z=\pi/2)$')
            plt.legend()
    
            zind = 4
            plt.subplot(3,5,5)
            plt.title(spec+' '+r'$|T_{\perp}|^2(\Delta x, \Delta y)(avg)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['T_perp_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,10)
            plt.plot(self.taux,np.sum(self.corr_funcs['T_perp_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['T_perp_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['T_perp_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{/perp}|^2(\Delta x)(z=avg)$')
            plt.legend()
            plt.subplot(3,5,15)
            plt.plot(self.tauy,np.sum(self.corr_funcs['T_perp_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['T_perp_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['T_perp_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|T_{/perp}|^2(\Delta y)(z=avg)$')
            plt.legend()
            plt.tight_layout()
            plt.show()

            #u_par
            plt.figure(figsize = (18,8))
            zind = 0
            plt.subplot(3,5,1)
            plt.title(spec+' '+r'$|u_{||}|^2(\Delta x, \Delta y)(z=-\pi)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['u_par_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,6)
            plt.plot(self.taux,np.sum(self.corr_funcs['u_par_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['u_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['u_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|u_{||}^2(\Delta x)(z=-\pi)$')
            plt.legend()
            plt.subplot(3,5,11)
            plt.plot(self.tauy,np.sum(self.corr_funcs['u_par_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['u_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['u_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|u_{||}|^2(\Delta y)(z=-\pi)$')
            plt.legend()
    
            zind = 1
            plt.subplot(3,5,2)
            plt.title(spec+' '+r'$|u_{||}|^2(\Delta x, \Delta y)(z=-\pi/2)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['u_par_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,7)
            plt.plot(self.taux,np.sum(self.corr_funcs['u_par_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['u_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['u_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|u_{||}|^2(\Delta x)(z=-\pi/2)$')
            plt.legend()
            plt.subplot(3,5,12)
            plt.plot(self.tauy,np.sum(self.corr_funcs['u_par_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['u_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['u_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|u_{||}|^2(\Delta y)(z=-\pi/2)$')
            plt.legend()
    
            zind = 2
            plt.subplot(3,5,3)
            plt.title(spec+' '+r'$|u_{||}|^2(\Delta x, \Delta y)(z=0)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['u_par_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,8)
            plt.plot(self.taux,np.sum(self.corr_funcs['u_par_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['u_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['u_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|u_{||}|^2(\Delta x)(z=0)$')
            plt.legend()
            plt.subplot(3,5,13)
            plt.plot(self.tauy,np.sum(self.corr_funcs['u_par_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['u_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['u_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|u_{||}|^2(\Delta y)(z=0)$')
            plt.legend()
    
            zind = 3
            plt.subplot(3,5,4)
            plt.title(spec+' '+r'$|u_{||}|^2(\Delta x, \Delta y)(z=\pi/2)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['u_par_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,9)
            plt.plot(self.taux,np.sum(self.corr_funcs['u_par_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['u_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['u_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|u_{||}|^2(\Delta x)(z=\pi/2)$')
            plt.legend()
            plt.subplot(3,5,14)
            plt.plot(self.tauy,np.sum(self.corr_funcs['u_par_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['u_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['u_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|u_{||}|^2(\Delta y)(z=\pi/2)$')
            plt.legend()
    
            zind = 4
            plt.subplot(3,5,5)
            plt.title(spec+' '+r'$|u_{||}|^2(\Delta x, \Delta y)(avg)$')
            plt.contourf(self.tauy,self.taux,self.corr_funcs['u_par_'+spec][:,:,zind])
            plt.colorbar()
            plt.ylabel(r'$\Delta x(\rho_s)$')
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.subplot(3,5,10)
            plt.plot(self.taux,np.sum(self.corr_funcs['u_par_'+spec][:,:,zind],axis=1))
            plt.axvline(self.corr_lengths_x['u_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_x['u_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta x(\rho_s)$')
            plt.ylabel(spec+' '+r'$|u_{||}|^2(\Delta x)(z=avg)$')
            plt.legend()
            plt.subplot(3,5,15)
            plt.plot(self.tauy,np.sum(self.corr_funcs['u_par_'+spec][:,:,zind],axis=0))
            plt.axvline(self.corr_lengths_y['u_par_'+spec][zind],color = 'black',label=str(self.corr_lengths_y['u_par_'+spec][zind])[:6])
            plt.xlabel(r'$\Delta y(\rho_s)$')
            plt.ylabel(spec+' '+r'$|u_{||}|^2(\Delta y)(z=avg)$')
            plt.legend()
            plt.tight_layout()
            plt.show()


        
    def save(time_requested, output=None, out_folder=None):
        pass

def corr_func_2D(v1,v2,xgrid,ygrid,show_plot=False):
        dx=xgrid[1]-xgrid[0]
        dy=ygrid[1]-ygrid[0]
        Nx=len(xgrid)
        Ny=len(ygrid)
        cfunc=np.zeros((Nx,Ny))
        for i in range(Nx):
            for j in range(Ny):
                i0=i+1
                j0=j+1
                cfunc[-i0,-j0]=np.sum( np.sum( np.conj(v1[-i0:,-j0:])*v2[:i0,:j0],axis = 0),axis=0)
        cfunc = cfunc / np.sum(np.sum(np.conj(v1[:,:])*v2[:,:],axis = 0),axis=0)
        cfuncx = np.sum(cfunc,axis=1)
        cfuncy = np.sum(cfunc,axis=0)
        taux=np.arange(Nx)
        taux=taux*dx
        tauy=np.arange(Ny)
        tauy=tauy*dy
        corr_time_x = 0.0
        max_corr_x=max(np.abs(cfuncx))
        max_corr_y=max(np.abs(cfuncy))
        corr_time_y = 0.0
        i=0
        while corr_time_x==0.0:
            if (abs(cfuncx[i])-max_corr_x/np.e) > 0.0 and \
               (abs(cfuncx[i+1])-max_corr_x/np.e) <= 0.0:
                slope=(cfuncx[i+1]-cfuncx[i])/(taux[i+1]-taux[i])
                zero=cfuncx[i]-slope*taux[i]
                corr_time_x=(max_corr_x/np.e-zero)/slope
            i+=1
        i=0
        while corr_time_y==0.0:
            if (abs(cfuncy[i])-max_corr_y/np.e) > 0.0 and \
               (abs(cfuncy[i+1])-max_corr_y/np.e) <= 0.0:
                slope=(cfuncy[i+1]-cfuncy[i])/(tauy[i+1]-tauy[i])
                zero=cfuncy[i]-slope*tauy[i]
                corr_time_y=(max_corr_y/np.e-zero)/slope
            i+=1
        if show_plot:
            plt.title('taux: '+str(corr_time_x)[:6]+' tauy: '+str(corr_time_y)[:6])
            plt.contourf(tauy,taux,cfunc)
            plt.colorbar()
            plt.show()
            plt.plot(taux,cfuncx,'x-')
            ax=plt.axis()
            plt.vlines(corr_time_x,ax[2],ax[3])
            plt.show()
            plt.plot(tauy,cfuncy,'x-')
            ax=plt.axis()
            plt.vlines(corr_time_y,ax[2],ax[3])
            plt.show()

        return taux,tauy,cfunc


def corr_func_1D(v1,v2,grid,show_plot=True):
        dt=grid[1]-grid[0]
        N=len(grid)
        cfunc=np.zeros(N,dtype='complex')
        for i in range(N):
            i0=i+1
            cfunc[-i0]=np.sum(np.conj(v1[-i0:])*v2[:i0])
        tau=np.arange(N)
        tau=tau*dt
        max_corr=max(np.abs(cfunc))
        corr_time=0.0
        i=0
        while corr_time==0.0:
            if (abs(cfunc[i])-max_corr/np.e) > 0.0 and \
               (abs(cfunc[i+1])-max_corr/np.e) <= 0.0:
                slope=(cfunc[i+1]-cfunc[i])/(tau[i+1]-tau[i])
                zero=cfunc[i]-slope*tau[i]
                corr_time=(max_corr/np.e-zero)/slope
            i+=1
        if show_plot:
            plt.plot(tau,cfunc,'x-')
            ax=plt.axis()
            plt.vlines(corr_time,ax[2],ax[3])
            plt.show()
        return cfunc,tau,corr_time

def corr_func_1D_full_domain(v1,v2,grid,show_plot=True):
        dt=grid[1]-grid[0]
        N=len(grid)
        print("N",N)
        cfunc=np.zeros(2*N-1)
        cfunc[N-1]=np.sum(np.conj(v1[:])*v2[:])
        for i in range(N-1):
            i0=i+1
            cfunc[-i0]=np.sum(np.conj(v1[-i0:])*v2[:i0])
            cfunc[N-1-i0]=np.sum(np.conj(v1[:-i0])*v2[i0:])
        tau=np.arange(2*N-1)
        tau=tau*dt
        tau = tau-tau[-1]/2.0
        max_corr=max(np.abs(cfunc))
        corr_time=0.0
        i=0
        while corr_time==0.0:
            if (abs(cfunc[i])-max_corr/np.e) > 0.0 and \
               (abs(cfunc[i+1])-max_corr/np.e) <= 0.0:
                slope=(cfunc[i+1]-cfunc[i])/(tau[i+1]-tau[i])
                zero=cfunc[i]-slope*tau[i]
                corr_time=(max_corr/np.e-zero)/slope
            i+=1
        if show_plot:
            plt.plot(tau,cfunc,'x-')
            ax=plt.axis()
            plt.vlines(corr_time,ax[2],ax[3])
            plt.show()
        print("corr_time",corr_time)
        return cfunc,tau,corr_time

def translate_edge_opt(zval,nz,edge_opt):
    """Given a value of z (e.g. pi/2), this routine returns the nearest z index for an edge_opt-ed zgrid given nz0 and edge_opt."""
    dz = 2.0*np.pi/nz
    zprime_even = np.arange(nz)/float(nz-1)*(2.0*np.pi-dz)-np.pi
    N = np.arcsinh(edge_opt*zprime_even[-1])/zprime_even[-1]
    if edge_opt == 0.0:
        zind = np.argmin(abs(zprime_even-zval))
    else:
        zgrid = 1/edge_opt*np.sinh(N*zprime_even)
        zind = np.argmin(abs(zgrid-zval))
    return zind



import numpy as np
import ..utils.averages as av
from .baseplot import Plotting, plt
from .utils import errors as err


class PlotNrgdata(Plotting):
    """ PlotNrgdata: Class to generate plots from nrgdata objects

    :param nrgdata: List of NrgData objects to plot
    """
    def __init__(self, nrgdata):
        super().__init__()
        for nrgd in nrgdata:
            nrgd.generate_timeseries()
            if len(nrgd.dataarray) == 0:
                print("No nrg data present in object ", nrgd.cm.fileextension)
                # Names of the columns in LaTeX readable form
        self.colnames = [r"|n_1|^2", r"|u_{1,\parallel}|^2", r"|T_{1,\parallel}|^2",
                         r"|T_{1,\perp}|^2", r"\Gamma_{es}", r"\Gamma_{em}", r"Q_{es}",
                         r"Q_{em}", r"\Pi_{es}", r"\Pi_{em}"]
        self.nrgdata = nrgdata
        self.n_col1 = 4
        self.n_col2 = 6
        if self.n_col1+self.n_col2 != nrgdata[0].cm.pnt.nrgcols:
            raise RuntimeError("The number of nrg columns in the Nrgdata class does not agree "
                               "with the current implementation in the plotting routine")

    def print_mean_err(self):
        """ Calculate and print the mean and error estimate for each nrg column """
        for nrgd in self.nrgdata:
            mean_nrg, err_nrg, ctimes_nrg = self._calc_mean_and_error(nrgd)
            print("Mean and errors for ", nrgd.objectname)
            for i_sp in range(nrgd.cm.pnt.n_spec):
                print("Species: ", i_sp, nrgd.cm.specnames[i_sp])
                print("============================")
                for col in range(nrgd.cm.pnt.nrgcols):
                    print(self.colnames[col]+": ", mean_nrg[i_sp, col], "+-", err_nrg[i_sp, col])
                print("corr times: ", ctimes_nrg[i_sp, :])
                print("============================")

    @staticmethod
    def _calc_mean_and_error(nrgd):
        mean_nrg = av.mytrapz(nrgd.dataarray, nrgd.timearray)
        err_nrg, ctimes_nrg = err.windowerr(nrgd.dataarray, nrgd.timearray)
        return mean_nrg, err_nrg, ctimes_nrg

    def plot_ttrace(self, poutput=False):
        """ Plot nrg file time traces with means and errors

        :param poutput: Generate pdf files of the plots as well
        """
        is_nonlinear = self._is_nrglist_nonlinear()
        fig1 = plt.figure()
        fig2 = plt.figure()
        max_spec = max([nrgd.cm.pnt.n_spec for nrgd in self.nrgdata])
        for nrgd in self.nrgdata:
            for i_sp in range(max_spec):
                ax1 = fig1.add_subplot(1, max_spec, i_sp+1)
                ax1.set_xlabel(r'$t\: [L_\mathrm{ref}/c_\mathrm{ref}]$')
                ax1.set_title(nrgd.cm.specnames[i_sp])
                ax2 = fig2.add_subplot(1, max_spec, i_sp+1)
                ax2.set_xlabel(r'$t\: [L_\mathrm{ref}/c_\mathrm{ref}]$')
                ax2.set_title(nrgd.cm.specnames[i_sp])
                if is_nonlinear:
                    self.plot_ttrace_nonlinear(ax1, ax2, nrgd, i_sp)
                else:
                    self.plot_ttrace_linear(ax1, ax2, nrgd, i_sp)
        ax1.legend()
        ax2.legend()
        fig1.tight_layout()
        fig2.tight_layout()
        if poutput == 1:
            fig1.savefig("nrg_ttrace1{}.pdf".format(self.nrgdata[0].cm.fileextension))
            fig2.savefig("nrg_ttrace2{}.pdf".format(self.nrgdata[0].cm.fileextension))

    def _is_nrglist_nonlinear(self):
        if np.all([nrgd.cm.nonlinear for nrgd in self.nrgdata]):
            return True
        elif np.all([not nrgd.cm.nonlinear for nrgd in self.nrgdata]):
            return False
        else:
            raise ValueError("Inconsistent combination of linear and nonlinear runs in nrgdata")

    def plot_ttrace_linear(self, ax1, ax2, nrgd, i_sp):
        ax1.set_yscale('log')
        ax2.set_yscale('log')
        self._find_rescalings(nrgd, i_sp)
        for col in range(self.n_col1):
            texlabel = r"$" + self.colnames[col] + r"$"
            base_line, = ax1.plot(nrgd.timearray, nrgd.dataarray[:, i_sp, col],
                                  label=texlabel)
        for col in range(self.n_col1, self.n_col1 + self.n_col2):
            texlabel = r"$" + self.colnames[col] + r"$"
            ax2.plot(nrgd.timearray, nrgd.dataarray[:, i_sp, col], label=texlabel)
        ax1.set_ylim(bottom=1e-3)
        ax2.set_ylim(bottom=1e-3)
        self._estimate_growthrate(nrgd, i_sp)

    @staticmethod
    def _find_rescalings(nrgd, i_sp):
        """ Find rescalings in linear runs"""
        col = 3  # Tperp should be a good quantity for this
        trace = nrgd.dataarray[:,  i_sp, col]
        rel = (trace/np.roll(trace, -1))[:-1]  # Find a large jump down between neighbouring points
        # Ignore the last element of rel, since it is trace[0]/trace[-1]
        return np.where(np.logical_and(rel > 1e12, trace[:-1] > 0.05))[0]

    def _estimate_growthrate(self, nrgd, i_sp):
        resc = self._find_rescalings(nrgd, i_sp)
        regrcolumns = [2]
        if len(resc) >= 2:
            start = resc[-2]+1    # Take time between the last two rescalings
            end = resc[-1] - 2    # Shift index by one to avoid including the rescaling
        elif len(resc) == 1:
            start = resc[0]+1     # Take time trace after rescaling
            end = None
        else:
            start = 0        # Take full time trace
            end = None
        for icol in regrcolumns:
            trace = nrgd.dataarray[start:end, i_sp, icol]/nrgd.dataarray[int(start), i_sp, icol]
            time = nrgd.timearray[start:end]-nrgd.timearray[start]
            growthrate = (np.log(trace[-1])-np.log(trace[0]))/(time[-1]-time[0])
            print("Growthrate estimate for {} {}: {}".format(nrgd.cm.specnames[i_sp],
                                                             self.colnames[icol], growthrate))

    def plot_ttrace_nonlinear(self, ax1, ax2, nrgd, i_sp):
        mean_nrg, err_nrg, ctimes_nrg = self._calc_mean_and_error(nrgd)

        # Plot first group: density, temperature, parallel flow etc
        for col in range(self.n_col1):
            texlabel = r"$" + self.colnames[col] + r"$"
            base_line, = ax1.plot(nrgd.timearray, nrgd.dataarray[:, i_sp, col],
                                  label=texlabel)
            if err_nrg[i_sp, col] != 0.0:
                ax1.axhline(y=mean_nrg[i_sp, col], linestyle='--', color=base_line.get_color())
                ax1.axhspan(mean_nrg[i_sp, col] - err_nrg[i_sp, col],
                            mean_nrg[i_sp, col] + err_nrg[i_sp, col], alpha=0.3,
                            facecolor=base_line.get_color())
        # Plot second group: Fluxes
        for col in range(self.n_col1, self.n_col1 + self.n_col2):
            texlabel = r"$" + self.colnames[col] + r"$"
            base_line, = ax2.plot(nrgd.timearray, nrgd.dataarray[:, i_sp, col],
                                  label=texlabel)
            ax2.axhline(y=mean_nrg[i_sp, col], linestyle='--', color=base_line.get_color())
            ax2.axhspan(mean_nrg[i_sp, col] - err_nrg[i_sp, col],
                        mean_nrg[i_sp, col] + err_nrg[i_sp, col], alpha=0.3,
                        facecolor=base_line.get_color())



""" par_io.py: Contains the class to handle reading and writing of parameter files """
import re
from collections import namedtuple, OrderedDict
import numpy as np
import os

class Parameters:
    """ Parameters class:

    Converts a GENE parameters file to a python dictionary and vice versa
    Usually an instance of this object is used to create a named tuple with
    .asnamedtuple()
    """

    def __init__(self, in_folder, extension):
        # dictionary for parameters: {parameter: value}
        self.pardict = OrderedDict()
        # dictionary for recording the namelists the parameters belong to: {parameter: namelist}
        self.nmldict = OrderedDict()
        # keep track of all namelists that have been found
        self.namelists = []
        self.spec_nl = (
            'omn', 'omt', 'mass', 'charge', 'dens', 'temp', 'name', 'passive', 'kappa_n', 'kappa_T',
            'LT_center', 'Ln_center', 'LT_width', 'Ln_width', 'prof_type', 'src_prof_type',
            'src_amp', 'src_width', 'src_x0', 'delta_x_n', 'delta_x_T', 'prof_file')
        self.specnames = []
        self.extension = extension
        self.in_folder = in_folder
        read_path = os.path.join( in_folder, 'parameters{}'.format(extension) )
        self.read_pars( read_path )
        self.pnt = self.asnamedtuple()

    @staticmethod
    def _clearcomments(variable):
        regex = re.compile(r'\s*([-+\'\"\[\];.,/a-zA-Z0-9_\s*]*)\s*!?\s*(.*)')
        result = regex.search(variable)
        if result and result.group(2)[:4] != 'scan':
            return regex.search(variable).group(1)
        else:
            return variable

    def read_pars(self, path):
        """ Read parameters file and make it a dict """
        self.pardict.clear()
        self.nmldict.clear()
        # counts species namelists
        countspec = 0
        ispec=-1
        self.species=[]
        try:
            with open(path, "r") as parfile:
                # Search file for parameters using regular expressions
                for line in parfile:
                    # Exclude commented lines
                    if re.search(r'\s*!\w*\s*=.*', line) is None:
                        # Check for and count species namelists
                        if re.search(r'^\s*&(.*)', line):
                            # if namelist belongs to a species, append its number to the namelist
                            if re.search(r'^\s*&(.*)', line).group(1) == 'species':
                                countspec += 1
                                nml = re.search(r'^\s*&(.*)', line).group(1) + str(countspec)
                            else:
                                nml = re.search(r'^\s*&(.*)', line).group(1)
                            if nml not in self.namelists:
                                self.namelists.append(nml)
                    # Search lines for <parameter> = <value> patterns
                    par = re.compile(r'^\s*(.*)\s*=\s*(.*)')
                    match = par.search(line)
                    # Pick matching lines and build parameter dictionary
                    if match:
                        # need to sort species by name and not by appending a number
                        if match.group(1).strip() in self.spec_nl:
                            # the first output of GENE is the name, so this should be always fine
                            if match.group(1).strip() == 'name':
                                myname = match.group(2).strip().replace("'", "")
                                ispec+=1
                                self.specnames.append(myname)
                                self.species.insert(ispec,{"name": myname})
                            else:
                                #cast all to doubles
                                self.species[ispec].update({match.group(1).strip(): np.array(match.group(2),dtype=np.float64)})
                            self.pardict[match.group(1).strip() + myname] = match.group(2)
                            self.nmldict[match.group(1).strip() + myname] = nml
                            
                        else:
                            self.pardict[match.group(1).strip()] = match.group(2)
                            self.nmldict[match.group(1).strip()] = nml


        except IOError:
            print("Could not read parameters file")
            raise
        self._clean_parameters()
        self.add_defaults()

    def _clean_parameters(self):
        """ Clear the comments from all variables,

        Cast some strings to integers and floats
        """
        boolstr_t = [".T.", ".t.", "T", "t", ".true."]
        boolstr_f = [".F.", ".f.", "F", "f", ".false."]
        for item in self.pardict:
            self.pardict[item] = self._clearcomments(self.pardict[item])
            try:  # Can it be converted to int?
                self.pardict[item] = int(self.pardict[item])
            except ValueError:
                try:  # No, but can it be converted to float?
                    self.pardict[item] = float(self.pardict[item])
                except ValueError:
                    pass
            if self.pardict[item] in boolstr_t:  # cast switches to boolean values
                self.pardict[item] = True
            elif self.pardict[item] in boolstr_f:
                self.pardict[item] = False

    def add_defaults(self):
        """ Set default values GENE does not write

        Some diagnostics require these to exist at least in the named tuple
        """
        # defpars = {"x0": 0.5, "ky0_ind": 0, "Tref": 1.0, "nref": 1.0, "Bref": 1.0, "mref": 1.0,
        #           "Lref": 1.0, "sign_Ip_CW": 1, "sign_Bt_CW": 1,}
        defpars = {"x0": 0.5, "ky0_ind": 0, "kx_center": 0.0, "n0_global": 0,
                   "lx" : 0.0, "mu_grid_type": "gau_lag",
                   "omega_prec": 1E-3,
                   "coll": 0.0, "Omega0_tor": 0.0, "ExBrate": 0.0, "with_coriolis": False,
                   "with_centrifugal": False, "with_comoving_other": False, "with_bxphi0": False,
                   "sign_Ip_CW": 1, 
                   "sign_Bt_CW": 1, 
                   "n_pol": 1, 
                   "Tref": 1.0, 
                   "nref": 1.0,
                   "Bref": 1.0, 
                   "mref": 1.999, 
                   "Lref": 1.0, 
                   "write_h5": False,
                   "write_adios": False,
                   "istep_srcmom": 0, 
                   "ck_heat": 0.0, 
                   "ck_part": 0.0,
                   "PRECISION": "DOUBLE",
                   "ENDIANNESS": "LITTLE"}
        defpnml = {"x0": "box", 
                   "ky0_ind": "box", 
                   "kx_center": "box",
                   "n0_global": "box",
                   "lx": "box",
                   "mu_grid_type": "box",
                   "omega_prec": "general",
                   "coll": "general", 
                   "istep_srcmom": "in_out", 
                   "write_h5": "in_out",
                   "write_adios": "in_out",
                   "Omega0_tor": "external_contr",
                   "ExBrate": "external_contr", 
                   "with_coriolis": "external_contr",
                   "with_centrifugal": "external_contr",
                   "with_comoving_other": "external_contr",
                   "with_bxphi0": "external_contr",
                   "sign_Ip_CW": "geometry",
                   "sign_Bt_CW": "geometry",
                   "n_pol": "geometry",
                   "Tref": "units",
                   "nref": "units",
                   "Bref": "units",
                   "mref": "units",
                   "Lref": "units", 
                   "ck_heat": "nonlocal_x", 
                   "ck_part": "nonlocal_x",
                   "PRECISION": "info",
                   "ENDIANNESS": "info"}
        for defkey in defpars:
            self.pardict.setdefault(defkey, defpars[defkey])
            self.nmldict.setdefault(defkey, defpnml[defkey])
        try:
            minor_r = self.pardict["minor_r"]
        except KeyError:
            minor_r = 1
        try:
            rhostar = self.pardict["rhostar"]
        except KeyError:
            rhostar = np.sqrt(
                self.pardict["Tref"]*self.pardict["mref"]*1.e3/1.60217733E-19*1.67262e-27)/ \
                  self.pardict["Bref"]/minor_r/self.pardict["Lref"]
            self.pardict["rhostar"] = rhostar
        if self.pardict.get("lx_a", 0.0) == 0.0 and self.pardict.get("lx", 0.0) == 0.0:
            raise ValueError('Cannot determine lx')
        if self.pardict.get("lx", 0.0) == 0.0:
            self.pardict["lx"] = self.pardict["lx_a"] / rhostar
        

        ################################
        ## these should all disappear ##
        ################################
        self.pardict['x_local'] = self.pardict['x_local'] if 'x_local' in self.pardict else True
        self.pardict['y_local'] = self.pardict['y_local'] if 'y_local' in self.pardict else True
        self.pardict['is3d'] = not self.pardict['x_local'] and not self.pardict['y_local']
        self.pardict['x_global'] = not self.pardict['x_local'] and self.pardict['y_local']
        self.pardict['flux_tube'] = self.pardict['x_local'] and self.pardict['y_local']
        self.pardict['is_adios'] = self.pardict['write_hac'] if 'write_hac' in self.pardict else False
        self.pardict['electromagnetic'] = (self.pardict.get('beta', False) != 0)
        self.pardict['bpar'] = self.pardict['bpar'] if 'bpar' in self.pardict else False
        self.pardict['specnames'] = self.specnames

    def write_pars(self, path):
        """ Take the dict and write a GENE parameters file """
        parfileout = open(path, "w")
        specflag = False
        for item in self.namelists:
            specflag = False
            if item[0:-1] == 'species':
                specflag = True
                parfileout.write('&' + item[0:-1] + '\n')
            else:
                parfileout.write('&' + item + '\n')
            for par in self.pardict.keys():
                if self.nmldict[par] == item:
                    self._writeparameter(par, parfileout, specflag)
            parfileout.write('/\n\n')

    def _writeparameter(self, par, parfile, specflag):
        parvalue = str(self.pardict[par])
        # Take care of the different form of logic states in Python and Fortran
        if parvalue == "True":
            parvalue = ".T."
        elif parvalue == "False":
            parvalue = ".F."
        if specflag:
            # Get rid of the species names at the end
            for sname in self.specnames:
                strippar = par.replace(sname, "")
            parfile.write(strippar + ' = ' + parvalue + '\n')
        else:
            parfile.write(par + ' = ' + parvalue + '\n')

    def asnamedtuple(self):
        """ Return Parameters as a named tuple for easier usage """
        # We have to ignore parameters with whitespace (in info nml)
        valid = {}
        for item in self.pardict:
            if " " not in item:
                valid[item] = self.pardict[item]
        return namedtuple('ParTuple', valid.keys())(*valid.values())